import tkinter as tk
from tkinter import ttk

from .layer_painter import LayerPainter


class LayerGrid(tk.Frame):
    """
    Represents the paintable canvas associated with one layer. One is contained
    inside each tab.
    """
    def __init__(self, zlayer, master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        self.zlayer = zlayer
        self.paintarea = LayerPainter(self)
        self.paintarea.grid(column=0, row=0)


class Drawarea(tk.Frame):
    """
    The main area of the window where drawing takes place.
    """
    def __init__(self, master=None):
        super().__init__(master, relief="sunken", borderwidth=1)
        self.master = master
        self.grid(column=0, row=0)

        self.tabs = ttk.Notebook(self)
        self.tabs.grid(column=0, row=0)
        self.layers = []
        self.zoomvalues = [5, 7, 10, 15, 20]
        self.zoom = 2

        self.tool = 'pencil'

        self.molecule = None

        self.actions = tk.Menu(self)
        self.actions.add_command(label="Modify Layer", command=self.master.cmds.edit_layer_action)
        self.actions.add_command(label="Delete Layer", command=self.master.cmds.delete_layer_action)
        self.tabs.bind('<<NotebookTabChanged>>', self.count_molecules)
        self.tabs.bind('<Control-1>', self.show_actions)
        self.tabs.bind('<2>', self.show_actions)
        self.tabs.bind('<3>', self.show_actions)

    def count_molecules(self, event):
        cur_tab = self.tabs.nametowidget(self.tabs.select())
        cur_tab.paintarea.count_molecules()

    def show_actions(self, event):
        cur_layer = self.tabs.nametowidget(self.tabs.select()).zlayer
        self.master.cmds.newlayer = cur_layer
        self.actions.entryconfigure(0, label='Edit "{}"'.format(cur_layer.name))
        self.actions.entryconfigure(1, label='Delete "{}"'.format(cur_layer.name))
        self.actions.post(event.x_root, event.y_root)

    def add_layer(self, zlayer):
        """
        Add a tab for this layer.
        """
        newlayer = LayerGrid(zlayer=zlayer, master=self.tabs)
        self.layers.append(newlayer)
        self.tabs.add(newlayer, text=newlayer.zlayer.name)

    def forget_layer(self, zlayer):
        """
        Find the tab associated with this layer and remove it. Call this before
        deleting a layer.
        """
        for tabid in self.tabs.tabs():
            if self.tabs.nametowidget(tabid).zlayer == zlayer:
                self.tabs.forget(tabid)
        
    def forget_all_layers(self):
        """
        Delete all tabs - can be used when creating a new project
        """
        for tabid in self.tabs.tabs():
            self.tabs.forget(tabid)
        
    def refresh_tabs(self):
        """
        Makes every tab display the name of the associated layer. Call this after
        changing a layer name so that the tabs match.
        """
        for tabid in self.tabs.tabs():
            self.tabs.tab(tabid, text=self.tabs.nametowidget(tabid).zlayer.name)
            self.tabs.nametowidget(tabid).paintarea.count_molecules()

        for layer in self.layers:
            layer.paintarea.refresh_image()

    def zoom_in(self):
        """
        Increase zoom level, if possible
        """
        if self.zoom < len(self.zoomvalues)-1:
            self.zoom += 1
            self.refresh_tabs()

    def zoom_out(self):
        """
        Decrease zoom level, if possible
        """
        if self.zoom > 0:
            self.zoom -= 1
            self.refresh_tabs()

    def set_tool(self, tool):
        """
        Change what tool will be used to paint on the layer
        """
        self.tool = tool
        for layer in self.layers:
            if tool == 'pencil':
                layer.paintarea.canvas.config(cursor="pencil")
            elif tool == 'rect':
                layer.paintarea.canvas.config(cursor="cross")
            elif tool == 'spray':
                layer.paintarea.canvas.config(cursor="spraycan")
        



import tkinter as tk

from .project import Project
from .commands import Commands
from .cmdbar import CmdBar
from .toolbox import Toolbox
from .molecule_lister import MoleculeLister
from .drawarea import Drawarea


class Painter():
    def __init__(self):
        self.project = Project()

        self.root = tk.Tk()
        self.root.title("MolPainter")
        self.root.option_add('*tearOff', False)
        self.gui = PainterWindow(project=self.project, master=self.root)

        self.gui.cmds.new_project_action()

        self.gui.mainloop()


class PainterWindow(tk.Frame):
    def __init__(self, project, master=None):
        super().__init__(master)
        self.project = project
        self.master = master
        self.grid(column=0, row=0)

        self.cmds = Commands(gui=self, project=project)
        self.cmdbar = CmdBar(self)
        self.cmdbar.grid(column=0, row=0, columnspan=9, rowspan=1, sticky=("W", "N"))
        self.toolbox = Toolbox(self)
        self.toolbox.grid(column=0, row=1, columnspan=1, rowspan=7, sticky=("W", "N"))
        self.mlist = MoleculeLister(self)
        self.mlist.grid(column=0, row=8, columnspan=1, rowspan=9, sticky=("S", "W", "N"))
        self.drawarea = Drawarea(self)
        self.drawarea.grid(column=1, row=1, columnspan=8, rowspan=14, sticky=("E", "S", "W", "N"))

        # Start out with Pencil tool selected
        self.toolbox.btn_pencil.invoke()

    def layer_created(self, layer):
        """
        Used to notify the GUI that a new Z Layer was added
        """
        self.drawarea.add_layer(layer)
        if self.project.import_solute is not None:
            self.project.overlay_solute(layer)
            self.drawarea.refresh_tabs()

    def layer_modified(self, layer):
        """
        Used to notify the GUI that a Z Layer was modified
        """
        if self.project.import_solute is not None:
            self.project.overlay_solute(layer)
        self.drawarea.refresh_tabs()

    def layer_removed(self, layer):
        """ 
        Used to notify the GUI that a Z Layer will be deleted
        """
        self.drawarea.forget_layer(layer)

    def molecule_created(self, molecule):
        """
        Used to notify the GUI that a new molecule was added
        """
        self.mlist.add_molecule(molecule)

    def molecule_modified(self):
        """
        Used to notify the GUI that a molecule was modified
        """
        self.mlist.display_molecules()
        self.drawarea.refresh_tabs()

    def molecule_removed(self, molecule):
        """ 
        Used to notify the GUI that a molecule will be deleted
        """
        self.mlist.forget_molecule(molecule)

    def molecule_selected(self, molecule):
        """
        Used to notify the GUI that a molecule is selected
        """
        self.drawarea.molecule = molecule

    def blend_created(self, blend):
        """
        Used to notify the GUI that a new blender was added
        """
        self.mlist.add_blend(blend) 

    def blend_removed(self, blend):
        """ 
        Used to notify the GUI that a blend will be deleted
        """
        self.mlist.forget_blend(blend)

def main():
    painter = Painter()

if __name__ == "__main__":
    main()



import tkinter as tk
import numpy as np


class LayerPainter(tk.Frame):
    """
    This handles the painting actions
    """
    def __init__(self, master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        
        self.virtual_spacing = self.master.master.master.zoomvalues[self.master.master.master.zoom] # number of pixels defining width and height of lattice points in pixels
        self.hardpad = 0 #int(self.virtual_spacing/2 )

        self.canvas = tk.Canvas(self, width=(self.lattice_width*self.virtual_spacing)+self.hardpad,\
                        height=(self.lattice_height*self.virtual_spacing)+self.hardpad, borderwidth=0,\
                        background="white", cursor="pencil")
        self.canvas.grid(column=0, row=0, padx=10, pady=10)

        # initialize the lattice as zeros -- this may be a list of lists forming a 2D array
        # will get these from elsewhere
        self.lattice = np.array([[0 for _ in range(self.lattice_width)] for _ in range(self.lattice_height)])
        # make the image we will draw on
        self.rectangles_image = tk.PhotoImage(master=self, width=(self.lattice_width*self.virtual_spacing)+self.hardpad,\
                                                    height=(self.lattice_height*self.virtual_spacing)+self.hardpad)
        # make the image all white, initially
        self.rectangles_image.put('white', to=(self.hardpad,self.hardpad,\
                                 int((self.lattice_width*self.virtual_spacing)+self.hardpad),\
                                 int((self.lattice_height*self.virtual_spacing)+self.hardpad)))
        # add the image to the canvas
        self.canvas.create_image(self.hardpad, self.hardpad, anchor="nw", image=self.rectangles_image)

        # make a grid of guiding lines, blocking out each 5x5 square
        #@TODO: Make this able to be turned on and off with a button
        self.fine_gridlines()
        self.guiding_gridlines()

        self.canvas.bind("<B1-Motion>", self.paint_action) # left click, dragged

        # bindings for selection rectangle, activated holding ctrl
        self.rectangle_active = False # if rectangle is being drawn, stop the pen!     
        self.canvas.bind("<Control-ButtonPress-1>", self.set_first_rectangle_coordinate)
        self.canvas.bind("<Control-ButtonRelease-1>", self.set_second_rectangle_coordinate_and_fill)
        # this is going to draw a rectangle to help guide the eyes during selection/drawing
        self.guiding_rectangle = None
        # spray can
        self.canvas.bind("<Shift-B1-Motion>", self.spraycan_on)
        # action completed
        self.canvas.bind("<ButtonRelease-1>", self.complete_action)

    @property
    def lattice_height(self):
        return self.master.master.master.master.project.lattice_height

    @property
    def lattice_width(self):
        return self.master.master.master.master.project.lattice_width

    @property
    def lattice_major_gridlines_index(self):
        return self.master.master.master.master.project.lattice_major_gridlines

    @property
    def spray_index(self):
        return self.master.master.master.master.toolbox.spray_width_scale.get()

    @property
    def active_tool(self):
        return self.master.master.master.tool

    def refresh_image(self):
        '''
        This function will redraw the layer image when called
        '''
        self.virtual_spacing = self.master.master.master.zoomvalues[self.master.master.master.zoom]
        self.canvas["width"] = (self.lattice_width*self.virtual_spacing)+self.hardpad
        self.canvas["height"] = (self.lattice_height*self.virtual_spacing)+self.hardpad
        self.rectangles_image["width"] = (self.lattice_width*self.virtual_spacing)+self.hardpad
        self.rectangles_image["height"] = (self.lattice_height*self.virtual_spacing)+self.hardpad
        self.canvas.delete('grid_line')
        self.rectangles_image.blank()
        self.rectangles_image.put('white', to=(self.hardpad,self.hardpad,\
                                 int((self.lattice_width*self.virtual_spacing)+self.hardpad),\
                                 int((self.lattice_height*self.virtual_spacing)+self.hardpad)))
        
        for row in range(self.master.zlayer.lattice.shape[0]):
            for col in range(self.master.zlayer.lattice.shape[1]):
                color = "#FFFFFF"
                molid = self.master.zlayer.lattice[row][col]
                if molid == -1:
                    # Obstructed cell
                    color = "#000000"
                else:
                    for mol in self.master.master.master.master.project.molecules:
                        if mol.index == molid:
                            color = mol.color
                            break
                self.rectangles_image.put(color, to=(int((col*self.virtual_spacing)),\
                      int((row*self.virtual_spacing)),\
                      int(((col+1)*self.virtual_spacing)),\
                      int(((row+1)*self.virtual_spacing))))
        self.fine_gridlines()
        self.guiding_gridlines()
        # cursor update
        self.set_cursor()

    def fine_gridlines(self):
        for i in range(0, self.lattice_width):
            # self.rectangles_image.put('black', to=((i*self.virtual_spacing), 0,\
            #                             (i*self.virtual_spacing) + 1, (self.lattice_height*self.virtual_spacing)))
            self.canvas.create_line([((i*self.virtual_spacing)+self.hardpad, 0),\
                ((i*self.virtual_spacing)+self.hardpad, (self.lattice_height*self.virtual_spacing)+self.hardpad)], tag='grid_line', width=1)
        for i in range(0, self.lattice_height):
            # self.rectangles_image.put('black', to=(0, (i*self.virtual_spacing),\
            #                             (self.virtual_spacing*self.lattice_width), (i*self.virtual_spacing) + 1))
            self.canvas.create_line([(0, (i*self.virtual_spacing)+self.hardpad),\
                ((self.virtual_spacing*self.lattice_width)+self.hardpad, (i*self.virtual_spacing)+self.hardpad)], tag='grid_line', width=1)
        return

    def guiding_gridlines(self):
        for i in range(0, self.lattice_width, self.lattice_major_gridlines_index):
            self.canvas.create_line([((i*self.virtual_spacing)+self.hardpad, 0),\
                ((i*self.virtual_spacing)+self.hardpad, (self.lattice_height*self.virtual_spacing)+self.hardpad)], tag='grid_line', width=2)
        for i in range(0, self.lattice_height, self.lattice_major_gridlines_index):
            self.canvas.create_line([(0, (i*self.virtual_spacing)+self.hardpad),\
                ((self.virtual_spacing*self.lattice_width)+self.hardpad, (i*self.virtual_spacing)+self.hardpad)], tag='grid_line', width=2)
        return

    def set_molecule(self, row, col):
        """
        Paint a single molecule onto this layer
        """
        if self.master.zlayer.lattice[row][col] == -1:
            # This is an obstructed site and can't be painted over
            return
        id, color = self.master.master.master.molecule.get_paint_props()
        if self.master.zlayer.lattice[row][col] != id:
            self.master.zlayer.lattice[row][col] = id
            self.rectangles_image.put(color, to=(int((col*self.virtual_spacing)),\
                                                int((row*self.virtual_spacing)),\
                                                int(((col+1)*self.virtual_spacing)),\
                                                int(((row+1)*self.virtual_spacing))))

    def count_molecules(self):
        number_of_molecules_in_layer = len(np.where(self.master.zlayer.lattice > 0)[0])
        molecules = self.master.master.master.master.mlist.molecules
        for i in range(1,len(molecules)):
            number_of_molecule = len(np.where(self.master.zlayer.lattice == molecules[i]["molecule"].index)[0])
            molecules[i]["number_tkvar"].set(str(number_of_molecule))
            if number_of_molecules_in_layer > 0:
                percent_of_molecule = str(np.around((number_of_molecule / number_of_molecules_in_layer)*100, decimals=1))+"%"
                molecules[i]["percent_tkvar"].set(percent_of_molecule)
            else:
                molecules[i]["percent_tkvar"].set("0.0%")
        return

    def set_cursor(self):
        """
        Make the cursor show the active tool
        """
        if self.active_tool == 'pencil':
            self.canvas.config(cursor="pencil")
        elif self.active_tool == 'rect':
            self.canvas.config(cursor="cross")
        elif self.active_tool == 'spray':
            self.canvas.config(cursor="spraycan")

    def paint_action(self, event):
        """
        Handle clicks on the canvas by painting with the currently selected tool.
        """
        if self.rectangle_active:
            self.draw_guiding_rectangle(event=event)
            return
        if self.active_tool == 'pencil':
            self.canvas.config(cursor="pencil")
            self.pencil(event=event)
        elif self.active_tool == 'rect':
            self.canvas.config(cursor="cross")
            if not self.rectangle_active:
                self.set_first_rectangle_coordinate(event=event)
        elif self.active_tool == 'spray':
            self.canvas.config(cursor="spraycan")
            self.spraycan_on(event=event)

    def spraycan_on(self, event):
        self.canvas.config(cursor="spraycan")
        spray_width = self.virtual_spacing*self.spray_index

        # within the square inscribed by minx, maxs...
        min_spray_x, max_spray_x = (event.x-self.hardpad)-spray_width, (event.x-self.hardpad)+spray_width
        min_spray_y, max_spray_y = (event.y-self.hardpad)-spray_width, (event.y-self.hardpad)+spray_width
        # spray paint all lattice points whose centers fall inside of spray_width from the click point
        rows = range(int(min_spray_y//self.virtual_spacing),int(max_spray_y//self.virtual_spacing)+1)
        cols = range(int(min_spray_x//self.virtual_spacing),int(max_spray_x//self.virtual_spacing)+1)

        # first compute and store polar radii (in units of lattice spacing) for points within the spray
        polar_radii = np.zeros((len(rows),len(cols)))
        for i in range(len(rows)):
            for j in range(len(cols)):
                polar_radii[i][j] = ( ((rows[i] + 0.5) - ((event.y-self.hardpad)/self.virtual_spacing))**2 + ((cols[j] + 0.5) - ((event.x-self.hardpad)/self.virtual_spacing))**2)

        # now paint the points from the spray that fall within the spray index
        for i in range(len(rows)):
            for j in range(len(cols)):
                row = rows[i]
                col = cols[j]
                if polar_radii[i][j] < self.spray_index**2:
                    # wrap around the PBC!
                    row = row % self.lattice_height
                    col = col % self.lattice_width
                    self.set_molecule(row, col)
        self.count_molecules()

    def complete_action(self, event):
        """
        Handle the release so the rectangle can complete
        """
        if self.rectangle_active:
            self.set_second_rectangle_coordinate_and_fill(event=event)
        self.set_cursor()

    def draw_guiding_rectangle(self, event):
        self.canvas.coords(self.guiding_rectangle, self.first_rectangle_x, self.first_rectangle_y, event.x, event.y)

    def set_first_rectangle_coordinate(self, event):
        self.rectangle_active = True # this will perclude the pen from activating at the same time
        self.canvas.config(cursor="cross")
        self.first_rectangle_x = event.x
        self.first_rectangle_y = event.y
        self.guiding_rectangle = self.canvas.create_rectangle(self.first_rectangle_x, self.first_rectangle_y,\
                         self.first_rectangle_x, self.first_rectangle_y, fill=None, outline='#00ff00', width=2)


    def set_second_rectangle_coordinate_and_fill(self, event):
        self.rectangle_active = False # this will allow the pen to be activated again
        self.second_rectangle_x = event.x
        self.second_rectangle_y = event.y
        self.canvas.delete(self.guiding_rectangle) # delete the guiding rectangle
        self.fill_selected_rectangle()
        self.set_cursor()

    def fill_selected_rectangle(self):
        # higher indices are toward the bottom right, lower indices are toward the top left
        col_first = int((self.first_rectangle_x-self.hardpad)//self.virtual_spacing)
        row_first = int((self.first_rectangle_y-self.hardpad)//self.virtual_spacing)
        col_second = int((self.second_rectangle_x-self.hardpad)//self.virtual_spacing)
        row_second = int((self.second_rectangle_y-self.hardpad)//self.virtual_spacing)

        if (col_first > col_second) and (row_first > row_second):
            rows = range(row_second, row_first+1)
            cols = range(col_second, col_first+1)
        elif (col_second > col_first) and (row_second > row_first):
            rows = range(row_first, row_second+1)
            cols = range(col_first, col_second+1)
        elif (col_second > col_first) and (row_first > row_second):
            rows = range(row_second, row_first+1)
            cols = range(col_first, col_second+1)
        elif (col_first > col_second) and (row_second > row_first):
            rows = range(row_first, row_second+1)
            cols = range(col_second, col_first+1)
        elif (col_first > col_second) and (row_first == row_second):
            cols = range(col_second, col_first+1)
            rows = [row_first]
        elif (col_second > col_first) and (row_first == row_second):
            cols = range(col_first, col_second+1)
            rows = [row_first]
        elif (col_first == col_second) and (row_first > row_second):
            cols = [col_first]
            rows = range(row_second, row_first+1)
        elif (col_first == col_second) and (row_second > row_first):
            cols = [col_first]
            rows = range(row_first, row_second+1)
        else: return # no boxes have been encapsulated, do nothing
        # let the selection wrap around the PBC
        rows = [row % self.lattice_height for row in rows]
        cols = [col % self.lattice_width for col in cols]
        for row in rows:
            for col in cols:
                self.set_molecule(row, col)
        self.count_molecules()

    def pencil(self, event):
        # Calculate column and row number
        col = int((event.x-self.hardpad)//self.virtual_spacing)
        row = int((event.y-self.hardpad)//self.virtual_spacing)
        # avoid attempting to under- or over-paint
        if (col < 0) or (row < 0): return
        if (col > self.lattice_width-1) or (row > self.lattice_height-1): return

        # If the tile is not filled, create a rectangle
        self.set_molecule(row, col)
        self.count_molecules()



import tkinter as tk
from tkinter import ttk

import numpy as np


class MoleculeLister(tk.Frame):
    """
    This defines the list of molecules and blends shown at the side of the window.
    """
    def __init__(self, master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0, padx=5, pady=5)
        self.canvas = tk.Canvas(self, borderwidth=1, relief="solid", background="#FFFFFF")
        self.canvas.grid(column=0, row=0, padx=1, pady=1, sticky=("E", "S", "W", "N"))
        self.mlist = tk.Frame(self.canvas, background="#FFFFFF")
        self.mlist.grid(column=0, row=0)
        
        self.selected = tk.IntVar()
        self.selected.set(0)
        self.selected.trace_variable("w", self.activate_molecule)

        self.molecules = []
        self.blends = []

        self.vscroll = ttk.Scrollbar(self, orient="vertical", command=self.canvas.yview)
        self.canvas.configure(yscrollcommand=self.vscroll.set)

        self.actions = tk.Menu(self)
        self.actions.add_command(label="Modify Molecule", command=self.master.cmds.edit_molecule_action)
        self.actions.add_command(label="Copy Molecule", command=self.master.cmds.copy_molecule_action)
        self.actions.add_command(label="Delete Molecule", command=self.master.cmds.delete_molecule_action)

    def show_actions(self, event, molecule):
        """
        Display a context menu for the molecule that was clicked. The empty
        molecule can not be modified or deleted. If a blend was clicked, set
        the menu actions to modify the blend.
        """
        state="normal"
        if molecule.index == 0:
            state="disabled"
            edit_cmd=None
            copy_cmd=None
            del_cmd=None
        elif molecule.index < self.master.project.blender_offset:
            self.master.cmds.newmolecule = molecule
            edit_cmd = self.master.cmds.edit_molecule_action
            copy_cmd = self.master.cmds.copy_molecule_action
            del_cmd = self.master.cmds.delete_molecule_action
        else:
            self.master.cmds.newblend = molecule
            edit_cmd = self.master.cmds.edit_blend_action
            copy_cmd = self.master.cmds.copy_blend_action
            del_cmd = self.master.cmds.delete_blend_action

        self.actions.entryconfigure(0, label='Edit "{}"'.format(molecule.name), state=state, command=edit_cmd)
        self.actions.entryconfigure(1, label='Copy "{}"'.format(molecule.name), state=state, command=copy_cmd)
        self.actions.entryconfigure(2, label='Delete "{}"'.format(molecule.name), state=state, command=del_cmd)
        
        self.actions.post(event.x_root, event.y_root)

    def add_molecule(self, molecule):
        """
        Adds this molecule to the list. Call this after creating a molecule.
        """
        number_tkvar = tk.StringVar()
        percent_tkvar = tk.StringVar()
        number_tkvar.set('0')
        percent_tkvar.set('0.0%')
        self.molecules.append({
            "color": tk.Canvas(self.mlist, width=20, height=20, background=molecule.color),
            "entry": tk.Radiobutton(self.mlist, text=str(molecule.name), variable=self.selected, value=molecule.index, background="#FFFFFF"),
            "number_tkvar": number_tkvar,
            "percent_tkvar": percent_tkvar,
            "number": tk.Label(self.mlist, textvariable=number_tkvar, background="#FFFFFF", width=5),
            "percent": tk.Label(self.mlist, textvariable=percent_tkvar, background="#FFFFFF", width=5),
            "molecule": molecule
            })
        self.display_molecules()
        self.molecules[-1]["entry"].select()

    def forget_molecule(self, molecule):
        """
        Remove this molecule from the list. Call this before deleting a molecule.
        """
        for mol in self.molecules:
            if mol["molecule"] == molecule:
                self.molecules.remove(mol)
                if self.selected.get() == molecule.index:
                    self.molecules[0]["entry"].select()
                return

    def add_blend(self, blend):
        """
        Adds this blend definition to the list.
        """
        newblend = {
            "color": tk.Canvas(self.mlist, width=20, height=20, background="#FFFFFF"),
            "image": tk.PhotoImage(master=self.mlist, width=20, height=20),
            "entry": tk.Radiobutton(self.mlist, text=str(blend.name), variable=self.selected, value=blend.index, background="#FFFFFF"),
            "blend": blend
        }
        newblend["color"].create_image("0", "0", anchor="nw", image=newblend["image"])
        self.draw_preview(blend, newblend["image"])
        self.blends.append(newblend)
        self.display_molecules()
        self.blends[-1]["entry"].select()

    def draw_preview(self, blend, image):
        """
        Updates the colorbox for a blend with a weighted blend of colors.
        """
        for row in range(int(image["height"])):
            for col in range(int(image["width"])):
                image.put(blend.get_paint_props()[1], to=(row, col))

    def forget_blend(self, blend):
        """
        Remove this blend from the list. Call this before deleting a blend.
        """
        for b in self.blends:
            if b["blend"] == blend:
                self.blends.remove(b)
                if self.selected.get() == blend.index:
                    self.molecules[0]["entry"].select()
                return

    def display_molecules(self):
        """
        Refreshes the molecule list to show current info on all molecules. Call
        this after modifying any molecules.
        """
        for element in self.mlist.grid_slaves():
            element.grid_forget()
        molcount = len(self.molecules)
        for i in range(molcount):
            colorbox = self.molecules[i]["color"]
            label = self.molecules[i]["entry"]
            number = self.molecules[i]["number"]
            percent = self.molecules[i]["percent"]
            molecule = self.molecules[i]["molecule"]
            colorbox["background"] = molecule.color
            label["text"] = str(molecule.name)
            self.display_element(colorbox, row=i, col=0, molecule=molecule)
            self.display_element(label, row=i, col=1, molecule=molecule)
            if i == 0:
                continue
            else:
                self.display_element(number, row=i, col=2, molecule=molecule)
                self.display_element(percent, row=i, col=3, molecule=molecule)
        for i in range(len(self.blends)):
            colorbox = self.blends[i]["color"]
            label = self.blends[i]["entry"]
            blend = self.blends[i]["blend"]
            label["text"] = str(blend.name)
            self.draw_preview(self.blends[i]["blend"], self.blends[i]["image"])
            self.display_element(colorbox, row=i+molcount, col=0, molecule=blend)
            self.display_element(label, row=i+molcount, col=1, molecule=blend)

    def display_element(self, element, row, col, molecule):
        """
        Puts the individual components of a molecule list item on the screen
        along with a context menu that is aware of the related molecule.
        """
        element.grid(column=col, row=row, padx=0, pady=4, sticky="W")
        element.bind('<Control-1>', lambda e: self.show_actions(e, molecule))
        element.bind('<2>', lambda e: self.show_actions(e, molecule))
        element.bind('<3>', lambda e: self.show_actions(e, molecule))

    def activate_molecule(self, arg1, arg2, arg3):
        """
        Sets the active molecule to be the one that was just clicked. If it is
        a blend, treat it as a molecule.
        """
        for mol in self.molecules:
            if mol["molecule"].index == self.selected.get():
                self.master.molecule_selected(mol["molecule"])
                return
        for b in self.blends:
            if b["blend"].index == self.selected.get():
                self.master.molecule_selected(b["blend"])
                return



import MDAnalysis as mda
import numpy as np
from scipy.spatial import distance
import toml 
import argparse

def main():
    parser = argparse.ArgumentParser(description='Read in the TOML-format input file')
    parser.add_argument("-i", "--input", type=str, help='TOML-format input file')
    parser.add_argument("-centerc", action='store_true', help='Center system at (x,y,z) = (0,0,0) after solvation')
    parser.add_argument("-zeroz", action='store_true', help='Raise the system such that min(z) = solvent spacing / 2 after solvation')
    args = parser.parse_args()
    
    inputs = toml.load(args.input)
    
    if 'solute' not in inputs: raise Exception('solute missing from input file')
    if 'output' not in inputs: raise Exception('output missing from input file')
    if 'shape' not in inputs['solute_lattice']: raise Exception('shape of solute_lattice missing from input file')
    if 'width' not in inputs['solute_lattice']: raise Exception('width of solute_lattice missing from input file')
    if 'height' not in inputs['solute_lattice']: raise Exception('height of solute_lattice missing from input file')
    if 'spacing' not in inputs['solute_lattice']: raise Exception('spacing of solute_lattice missing from input file')
    if 'spacing' not in inputs['solvent_lattice']: raise Exception('spacing of solvent_lattice missing from input file')
    if 'buffer' not in inputs['solvent_lattice']: raise Exception('buffer of solvent_lattice missing from input file')
    if 'lower_z_position' not in inputs['solvent_lattice']: raise Exception('lower_z_position of solvent_lattice missing from input file')
    if 'upper_z_position' not in inputs['solvent_lattice']: raise Exception('upper_z_position of solvent_lattice missing from input file')
    if 'paths' not in inputs['solvent_molecules']: raise Exception('paths list of solvent_molecules missing from input file')
    if 'numbers' not in inputs['solvent_molecules']: raise Exception('numbers list of solvent_molecules missing from input file')
    if 'max_iterations' not in inputs['solvent_molecules']: raise Exception('max_iterations missing from input file')
    if 'rotate' not in inputs['solvent_molecules']: raise Exception('rotate missing from input file')
    if len(inputs['solvent_molecules']['paths']) != len(inputs['solvent_molecules']['numbers']):
        raise Exception('There should be an equal number of entries for paths and numbers of solent_molecules')
    
    # 1) Load the solute to be solvated into a universe
    u = mda.Universe(inputs['solute'])
    layers_positions = u.atoms.positions
    
    # 2) Define lattice_width and lattice_height coming from the canvas
    lattice_width = inputs['solute_lattice']['width'] # 32
    lattice_height = inputs['solute_lattice']['height'] # 16
    lattice_spacing = inputs['solute_lattice']['spacing'] # 8 angstroms
    
    # 3) Define solvent lattice spacing
    solvent_lattice_spacing = inputs['solvent_lattice']['spacing']# 5 angstroms
    solvent_lattice_buffer  = inputs['solvent_lattice']['buffer'] # 4 angstroms
    
    solvent_lattice_width = int((lattice_width*lattice_spacing)//solvent_lattice_spacing)
    if inputs['solute_lattice']['shape'] == 'hex': solvent_lattice_height = int(((lattice_height*lattice_spacing)+(lattice_spacing/2.))//solvent_lattice_spacing)
    elif inputs['solute_lattice']['shape'] == 'square': solvent_lattice_height = int((lattice_height*lattice_spacing)//solvent_lattice_spacing)
    solvent_z1 = inputs['solvent_lattice']['lower_z_position'] # 17 angstroms
    solvent_z2 = inputs['solvent_lattice']['upper_z_position'] # 72 angstroms
    solvent_lattice_depth = int(np.abs(solvent_z2 - solvent_z1)//solvent_lattice_spacing)
    
    # 4) Make and array to contain solvent molecule IDs, where "0" is Empty, generate the lattice
    # 3-dimensional array specifying molecule IDs of each lattice point
    solvent_lattice = np.zeros((solvent_lattice_width,solvent_lattice_height,solvent_lattice_depth)).astype('int')
    # 4-dimensional array specifying positions of each lattice point
    solvent_lattice_xyz_coordinates = np.zeros((solvent_lattice_width,solvent_lattice_height,solvent_lattice_depth,3))
    for i in range(solvent_lattice_width):
        for j in range(solvent_lattice_height):
            for k in range(solvent_lattice_depth):
                x = (i*solvent_lattice_spacing)-(solvent_lattice_spacing/2)
                y = (j*solvent_lattice_spacing)-(solvent_lattice_spacing/2)
                z = (k*solvent_lattice_spacing)+solvent_z1
                solvent_lattice_xyz_coordinates[i][j][k] = np.array([x,y,z])
    
    # 5) Mark cells in the array as -1000 if molecules in the layers are within "lattice_spacing" distance of points
    # Check to see if any lattice site is within solvent_lattice_buffer distance to the solute -- if it is, mark the cell with -1000
    solvent_lattice_sites = np.where(solvent_lattice == 0)
    for i in range(len(solvent_lattice_sites[0])):
        xyz = solvent_lattice_xyz_coordinates[solvent_lattice_sites[0][i]][solvent_lattice_sites[1][i]][solvent_lattice_sites[2][i]]
        if np.amin(distance.cdist(layers_positions, [xyz])) < solvent_lattice_buffer:
            solvent_lattice[solvent_lattice_sites[0][i]][solvent_lattice_sites[1][i]][solvent_lattice_sites[2][i]] = -1000
    
    def rand_rotation_matrix():
        # This is a pythonic implementation of the random 3D rotation method described
        # in the Graphics Gems book series by Jim Arvo in 1991
        # http://www.realtimerendering.com/resources/GraphicsGems/gemsiii/rand_rotation.c
        randnums = np.random.uniform(size=(3,))
        theta, phi, z = randnums
        theta = theta*2.0*np.pi  # Rotation about the pole (Z).
        phi   = phi*2.0*np.pi  # For direction of pole deflection.
        z     = z*2.0  # For magnitude of pole deflection.
    
        r = np.sqrt(z)
        V = (np.sin(phi) * r,
             np.cos(phi) * r,
             np.sqrt(2.0 - z))
        st = np.sin(theta)
        ct = np.cos(theta)
        R = np.array(((ct, st, 0), (-st, ct, 0), (0, 0, 1)))
        M = (np.outer(V, V) - np.eye(3)).dot(R)
        return M
    
    def create_solvent_universe(path, solvent_molecule_id, n_residues):
        solvu = mda.Universe(path)
        solvuxyz = np.copy(solvu.atoms.positions)
        solvent_positions = solvu.atoms.positions[np.where(solvu.atoms.tempfactors == 1)[0]]  
        solvent_cog = np.mean(solvent_positions, axis=0)
        solvuxyz = solvuxyz - solvent_cog
    
        mol_n_atoms = solvu.atoms.n_atoms
        n_atoms = mol_n_atoms*n_residues
        resindices = np.repeat(range(n_residues), mol_n_atoms)
        segindices = [0]*n_residues
    
        # initialize an empty universe and populate it
        solvent_universe = mda.Universe.empty(n_atoms, n_residues=n_residues, atom_resindex=resindices,
                                              residue_segindex=segindices, trajectory=True)
        solvent_universe.add_TopologyAttr('name', list(solvu.atoms.names)*n_residues)
        solvent_universe.add_TopologyAttr('type', list(solvu.atoms.types)*n_residues)
        solvent_universe.add_TopologyAttr('resnames', list(solvu.atoms.residues.resnames)*n_residues)
        solvent_universe.add_TopologyAttr('resid', list(range(1, (n_residues)+1)))
        solvent_universe.add_TopologyAttr('segid', [solvu.atoms.segids[0]])
        solvent_coordinates = np.concatenate(np.array([solvu.atoms.positions]*n_residues))
        return solvent_universe
    
    def lookup_lattice_xyz(lattice_xyz, lattice_sites, site_index):
        x, y, z = lattice_xyz[lattice_sites[0][site_index]][lattice_sites[1][site_index]][lattice_sites[2][site_index]]
        return x, y, z
    
    def assign_lattice_positions(path, solvent_lattice, solvent_molecule_id, rotation_matrices):
        solvu = mda.Universe(path)
        solvxyz = solvu.atoms.positions
        solvcom = solvu.atoms.center_of_mass()
    
        solvent_lattice_sites = np.array(np.where(solvent_lattice == solvent_molecule_id))
        # assign coordinates
        solvent_coordinates = []
        for i in range(len(solvent_lattice_sites[0])):
            xyz = np.copy(solvxyz - solvcom) # centered coordinates
            if inputs['solvent_molecules']['rotate'] == True:
                xyz = np.dot(xyz, rotation_matrices[i])
    
            x, y, z = lookup_lattice_xyz(solvent_lattice_xyz_coordinates, solvent_lattice_sites, i)
            xyz[:,0] += x
            xyz[:,1] += y
            xyz[:,2] += z
        
            solvent_coordinates.append(xyz)
        if len(solvent_lattice_sites[0]) > 1: solvent_coordinates = np.concatenate(solvent_coordinates)
        return solvent_coordinates
    
    def check_nearby_lattice_sites(n_range_x, n_range_y, n_range_z, col, row, dep, solvent_lattice):
        for x in n_range_x:
            for y in n_range_y:
                for z in n_range_z:
                    if solvent_lattice[col+x][row+y][dep+z] != 0:
                        return False
        return True
    
    def assign_empty_lattice_site(empty_lattice_sites):
        randnum = np.random.randint(0, high=len(empty_lattice_sites[0]))
        randcol = empty_lattice_sites[0,randnum]
        randrow = empty_lattice_sites[1,randnum]
        randdep = empty_lattice_sites[2,randnum]
        return randcol, randrow, randdep
    
    def insert_to_random_empty_point(path, solvent_lattice, solvent_molecule_id):
        solvu = mda.Universe(path)
        solvxyz = solvu.atoms.positions
        solvcom = solvu.atoms.center_of_mass()
        xyz = np.copy(solvxyz - solvcom) # centered coordinates
        if inputs['solvent_molecules']['rotate'] == True:
            rotation_matrix = rand_rotation_matrix()
            xyz = np.dot(xyz, rand_rotation_matrix())
        else: rotation_matrix = 0
    
        n_pos_x =  int(((abs(max(xyz[:,0]))+solvent_lattice_buffer) // solvent_lattice_spacing))
        n_neg_x = int(-((abs(min(xyz[:,0]))+solvent_lattice_buffer) // solvent_lattice_spacing))
        n_pos_y =  int(((abs(max(xyz[:,1]))+solvent_lattice_buffer) // solvent_lattice_spacing))
        n_neg_y = int(-((abs(min(xyz[:,1]))+solvent_lattice_buffer) // solvent_lattice_spacing))
        n_pos_z =  int(((abs(max(xyz[:,2]))+solvent_lattice_buffer) // solvent_lattice_spacing))
        n_neg_z = int(-((abs(min(xyz[:,2]))+solvent_lattice_buffer) // solvent_lattice_spacing))
        n_range_x = list(range(n_neg_x, n_pos_x+1))
        n_range_y = list(range(n_neg_y, n_pos_y+1))
        n_range_z = list(range(n_neg_z, n_pos_z+1))
    
        empty_lattice_sites = np.array(np.where(solvent_lattice == 0))
        # prune empty lattice sites that can't fit the excess space
        invalid_entries = []
        invalid_entries.append(np.where(empty_lattice_sites[0] <= -n_neg_x)[0])
        invalid_entries.append(np.where(empty_lattice_sites[1] <= -n_neg_y)[0])
        invalid_entries.append(np.where(empty_lattice_sites[2] <= -n_neg_z)[0])
        invalid_entries.append(np.where(empty_lattice_sites[0] >= (solvent_lattice.shape[0] - n_pos_x))[0])
        invalid_entries.append(np.where(empty_lattice_sites[1] >= (solvent_lattice.shape[1] - n_pos_y))[0])
        invalid_entries.append(np.where(empty_lattice_sites[2] >= (solvent_lattice.shape[2] - n_pos_z))[0])
        invalid_entries = np.unique(np.concatenate(invalid_entries))
        empty_lattice_sites = np.delete(empty_lattice_sites, invalid_entries, axis=1)
    
        if len(empty_lattice_sites[0]) == 0:
            raise Exception('Lattice full!')
        else:
            # roll random number to determine cell to try to insert to
            randcol, randrow, randdep = assign_empty_lattice_site(empty_lattice_sites)
            # check surrounding cells to ensure they are also empty. If not, re-roll
            iteration = 0
            while iteration < inputs['solvent_molecules']['max_iterations']:
                iteration += 1
                choice_boolean = check_nearby_lattice_sites(n_range_x, n_range_y, n_range_z, randcol, randrow, randdep, solvent_lattice)
                if choice_boolean == True:
                    for x in n_range_x:
                        for y in n_range_y:
                            for z in n_range_z:
                                if (x == 0) and (y == 0) and (z == 0):
                                    solvent_lattice[randcol][randrow][randdep] = solvent_molecule_id
                                else:
                                    solvent_lattice[randcol+x][randrow+y][randdep+z] = -solvent_molecule_id
                    break
                elif choice_boolean == False:
                    randcol, randrow, randdep = assign_empty_lattice_site(empty_lattice_sites)
            if iteration == inputs['solvent_molecules']['max_iterations']:
                raise Exception('Failed to insert to lattice after max_interations!')
            return solvent_lattice, rotation_matrix
    
    solvent_groups = []
    for i in range(len(inputs['solvent_molecules']['paths'])):
        solvent_molecule_id = i+1
        path = inputs['solvent_molecules']['paths'][i]
        num_solvent = inputs['solvent_molecules']['numbers'][i]
        print('Inserting %i molecules of '%(num_solvent) + path)
        solvent_universe = create_solvent_universe(path, solvent_molecule_id, num_solvent)
        rotation_matrices = []
        for i in range(num_solvent):
            solvent_lattice, rotation_matrix = insert_to_random_empty_point(path, solvent_lattice, solvent_molecule_id)
            rotation_matrices.append(rand_rotation_matrix())
    
        solvent_universe.atoms.positions = assign_lattice_positions(path, solvent_lattice, solvent_molecule_id, rotation_matrices)
        solvent_groups.append(solvent_universe.atoms)
    
    all_solvent_universe = mda.Merge(*solvent_groups)
    full_universe = mda.Merge(u.atoms, all_solvent_universe.atoms)
    
    # translate output coordinates if flags are defined
    if args.centerc == True:
        xyz = np.copy(full_universe.atoms.positions)
        xyz[:,0] -= np.mean(xyz[:,0])
        xyz[:,1] -= np.mean(xyz[:,1])
        xyz[:,2] -= np.mean(xyz[:,2])
        full_universe.atoms.positions = xyz
    if args.zeroz == True:
        xyz = np.copy(full_universe.atoms.positions)
        xyz[:,2] = (xyz[:,2] - np.amin(xyz[:,2])) + (solvent_lattice_spacing/2.)
        full_universe.atoms.positions = xyz
    
    xedge = np.amax(full_universe.atoms.positions[:,0]) - np.amin(full_universe.atoms.positions[:,0]) + solvent_lattice_spacing/2.
    yedge = np.amax(full_universe.atoms.positions[:,1]) - np.amin(full_universe.atoms.positions[:,1]) + solvent_lattice_spacing/2.
    zedge = np.amax(full_universe.atoms.positions[:,2]) - np.amin(full_universe.atoms.positions[:,2]) + solvent_lattice_spacing/2.
    
    full_universe.dimensions = np.array([xedge, yedge, zedge, 90., 90., 90.])
    full_universe.atoms.write(inputs['output'], bonds=None)

if __name__ == "__main__":
    main()



import tkinter as tk
from tkinter import ttk, messagebox

from .layer_configurator import LayerConfigurator
from .molecule_configurator import MoleculeConfigurator


class CmdBar(tk.Frame):
    """
    This defines the row of buttons across the top of the window.
    """
    def __init__(self, master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        self.colindex = 0

        self.btn_new = tk.Button(self, text="New", command=self.master.cmds.new_project_action)
        self.btn_open = tk.Button(self, text="Open Painting", command=self.master.cmds.load_project_action)
        self.btn_save = tk.Button(self, text="Save Painting", command=self.master.cmds.save_project_action)
        self.btn_export = tk.Button(self, text="Export System", command=self.master.cmds.export_action)
        self.btn_import = tk.Button(self, text="Import Solute", command=self.master.cmds.import_solute_action)
        self.btn_newmol = tk.Button(self, text="New Molecule", command=self.master.cmds.new_molecule_action)
        self.btn_newblend = tk.Button(self, text="New Blend", command=self.master.cmds.new_blend_action)
        self.btn_newlayer = tk.Button(self, text="New Layer", command=self.master.cmds.new_layer_action)
        self.btn_gridsize = tk.Button(self, text="Grid Settings", command=self.master.cmds.grid_setup_action)
        # self.btn_zoomout = tk.Button(self, text="Zoom Out", command=self.master.cmds.zoom_out_action)
        # self.btn_zoomin = tk.Button(self, text="Zoom In", command=self.master.cmds.zoom_in_action)
        #self.btn_solvents = tk.Button(self, text="Solvents", command=self.master.cmds.no_action)
        
        self.btn_new.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_open.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_save.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_export.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_import.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_newmol.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_newblend.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_newlayer.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        self.btn_gridsize.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        # self.btn_zoomout.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        # self.btn_zoomin.grid(column=self.col, row=0, columnspan=1, rowspan=1)
        # self.btn_solvents.grid(column=self.col, row=0, columnspan=1, rowspan=1)
    
    @property
    def col(self):
        self.colindex += 1
        return self.colindex



import os
import _thread
import tkinter as tk
from tkinter import filedialog
import numpy as np
import MDAnalysis as mda


class Exporter(tk.Frame):
    """
    This defines a window for creating a PDB structure out of the project contents.
    It contains any post-processing options, such as creating a hexagonal lattice.
    """
    def __init__(self, project, file, projlocation, master=None):
        super().__init__(master)
        self.master = master
        self.project = project
        self.grid(column=0, row=0)
        self.exportfilevar = file
        self.projfilevar = projlocation

        self.lblf = tk.Label(self, text="Path")
        self.lblf.grid(row=0, column=0, padx=2, pady=4)
        self.file_entry = tk.Entry(self, textvariable=self.exportfilevar)
        self.file_entry.grid(row=0, column=1, columnspan=3, padx=2, pady=4, sticky=("E", "W"))
        self.file_button = tk.Button(self, text="Browse", command=self.file_select_action)
        self.file_button.grid(row=0, column=4, padx=2, pady=4)

        self.lattice_choice = tk.StringVar()
        self.lattice_choice.set("Square") 

        self.square_button    = tk.Radiobutton(self, 
                                text="Square lattice",
                                padx = 2, 
                                variable=self.lattice_choice, 
                                value="Square")
        self.square_button.grid(row=1, column=0, padx=40, pady=4)
        self.hexagonal_button = tk.Radiobutton(self, 
                                text="Hexagonal lattice",
                                padx = 2, 
                                variable=self.lattice_choice, 
                                value="Hexagon")
        self.hexagonal_button.grid(row=1, column=1, padx=5, pady=4)

        self.cancel_button = tk.Button(self, text="Close", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="Export", command=self.ok_action)
        self.cancel_button.grid(row=2, column=0, padx=2, pady=4)
        self.ok_button.grid(row=2, column=1, padx=2, pady=4)

        self.statuslabel = tk.Label(self, text="Export status: not started.")
        self.statuslabel.grid(row=3, column=0, columnspan=5, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)
        self.lattice_spacing = self.project.lattice_spacing

    def file_select_action(self):
        self.exportfilevar.set(filedialog.asksaveasfilename(initialdir = "./",title = "Select file",filetypes = (("PDB file","*.pdb"),("all files","*.*"))))

    def cancel_action(self):
        self.master.destroy()

    def ok_action(self):
        self.ok_button["state"] = "disabled"
        export_thread = _thread.start_new_thread(self.start_export, ())

    def return_event(self, event):
        self.ok_button.invoke()

    def start_export(self):
        self.statuslabel["text"] = "Export status: started..."
        try:
            self.export_system()
        except Exception as e:
            self.statuslabel["text"] += " failed\n{}".format(e)

        self.ok_button["state"] = "normal"
        self.statuslabel["text"] += "\nStopped."

    def export_system(self):    
        layers = []
        if self.project.import_solute != None:
            layers.append(self.project.solute.atoms)
        for i in range(len(self.project.layers)):
            layers.append(self.constructionate_layer(self.project.layers[i]))
        layer = mda.Merge(*layers)
        # assign x and y dimensions to the box. The z-dimension will equal the smaller of the two.
        self.lattice_spacing = self.project.lattice_spacing
        x = (self.project.lattice_width*self.lattice_spacing)
        y = (self.project.lattice_height*self.lattice_spacing)
        # add 1 more lattice_spacing to each dimensions -- if one dim. is longer, add more
        if self.project.lattice_width > self.project.lattice_height:
            x += (float(self.project.lattice_width)/float(self.project.lattice_height))*(self.lattice_spacing/2.)
            y += self.lattice_spacing/2.
        elif self.project.lattice_height < self.project.lattice_width:
            x += self.lattice_spacing/2.
            y += (float(self.project.lattice_width)/float(self.project.lattice_height))*(self.lattice_spacing/2.)
        else:
            x += self.lattice_spacing/2.
            y += self.lattice_spacing/2.

        layer.dimensions = np.array([x, y, np.amin([x,y]), 90., 90., 90.])
        # numer all residue IDs from 1 up to the number of molecules
        layer.residues.resids = np.arange(1,layer.residues.n_residues+1)
        destfile = self.exportfilevar.get()
        layer.atoms.write(destfile)
        self.statuslabel["text"] += "\nOutput saved as {}".format(destfile)
        return

    def constructionate_layer(self, layer):
        if self.lattice_choice.get() == "Square":
            lattice_xy_coordinates = self.map_lattice_to_xy_squares()
        elif self.lattice_choice.get() == "Hexagon":
            lattice_xy_coordinates = self.map_lattice_to_xy_hexagons()

        all_pdb_molecule_groups = []
        for i in range(len(self.project.molecules)):
            molecule_id = self.project.molecules[i].index
            # swap the layer.lattice x,y coordinates, then flip the y-coordinates s.t. x > and y ^
            pdb_lattice_sites = np.where(np.flip(layer.lattice.swapaxes(0,1), axis=1) == molecule_id)
            # if there are 0 instances of this molecule, just continue
            if len(pdb_lattice_sites[0]) == 0:
                continue
            path = self.project.molecules[i].path
            if path == None: 
                continue
            if os.path.dirname(path) == '':
                path = os.path.join(os.path.dirname(self.projfilevar.get()), path)
            if not os.path.exists(path):
                continue
            pdbu = mda.Universe(path)
            if len(np.where(pdbu.atoms.tempfactors == 1)[0]) == 0:
                head_indices = pdbu.atoms.indices
            else:
                head_indices = np.where(pdbu.atoms.tempfactors == 1)[0]
            # create an empty Universe to store system parameters
            n_mols = len(pdb_lattice_sites[0]) # number of molecules
            mol_n_residues = pdbu.atoms.n_residues
            n_residues = n_mols*mol_n_residues
            mol_n_atoms = pdbu.atoms.n_atoms
            n_atoms = mol_n_atoms*n_mols
            # determine residue index of each atom
            resindices = []
            for j in range(n_mols):
                for k in range(mol_n_atoms):
                    resindices.append(pdbu.atoms.resindices[k]+(j*mol_n_residues))
            segindices = [0]*n_residues

            # initialize an empty universe and populate it
            molecule_universe = mda.Universe.empty(n_atoms, n_residues=n_residues, atom_resindex=resindices, 
                                                            residue_segindex=segindices, trajectory=True)
            # assign topology attributes
            molecule_universe.add_TopologyAttr('name', list(pdbu.atoms.names)*n_mols)
            molecule_universe.add_TopologyAttr('type', list(pdbu.atoms.types)*n_mols)
            molecule_universe.add_TopologyAttr('resnames', list(pdbu.atoms.residues.resnames)*n_mols)
            molecule_universe.add_TopologyAttr('resid', list(range(1, (n_residues)+1)))
            molecule_universe.add_TopologyAttr('segid', [list(pdbu.atoms.segids)[0]])
            molecule_coordinates = []

            if self.project.molecules[i].flipbool == 1:
                pdbu.atoms.positions = self.flip_molecule(pdbu.atoms.positions)
            pdbuxyz = np.copy(pdbu.atoms.positions)
            if 1 in pdbu.atoms.tempfactors:
                head_indices = np.where(pdbu.atoms.tempfactors == 1)[0]
                head_positions = pdbu.atoms.positions[head_indices]
                head_cog = np.mean(pdbu.atoms.positions[head_indices], axis=0)
            else:
                head_cog = np.mean(pdbu.atoms.positions, axis=0)
            pdbuxyz = pdbuxyz - head_cog # center the coordinates around the head
            pdbuxyz[:,2] += layer.zdepth # move the coordinates to zdepth such that the head is at zdepth


            for j in range(len(pdb_lattice_sites[0])):
                # in units angstroms
                y, x = lattice_xy_coordinates[pdb_lattice_sites[1][j]][pdb_lattice_sites[0][j]]
                pdbxyz = np.copy(pdbuxyz)
                if self.project.molecules[i].rotatebool == 1:
                    pdbxyz = self.rotate_molecule(pdbxyz)
                pdbxyz[:,0] += x
                pdbxyz[:,1] += y
                molecule_coordinates.append(pdbxyz)
            molecule_coordinates = np.concatenate(molecule_coordinates)
            molecule_universe.atoms.positions = molecule_coordinates
            all_pdb_molecule_groups.append(molecule_universe.atoms)
        return mda.Merge(*all_pdb_molecule_groups).atoms # this is an atomgroup for the layer

    def flip_molecule(self, xyz):
        R = np.array([[1,  0,  0],
                      [0, -1,  0],
                      [0,  0,  -1]])
        return np.dot(xyz,R)

    def rotate_molecule(self, xyz):
        theta = np.random.random()*2*np.pi
        R = np.array([[np.cos(theta), -np.sin(theta),  0],
                      [np.sin(theta), np.cos(theta),  0],
                      [0,  0,  1]])
        return np.dot(xyz,R)

    def map_lattice_to_xy_squares(self):
        self.lattice_spacing = self.project.lattice_spacing
        lattice_xy_coordinates = []
        for i in range(self.project.lattice_height):
            lattice_xy_coordinates_row = []
            for j in range(self.project.lattice_width):
                x = (i*self.lattice_spacing)+(self.lattice_spacing/2)
                y = (j*self.lattice_spacing)+(self.lattice_spacing/2)
                lattice_xy_coordinates_column = (x, y)
                lattice_xy_coordinates_row.append(lattice_xy_coordinates_column)
            lattice_xy_coordinates.append(lattice_xy_coordinates_row)
        return lattice_xy_coordinates

    def map_lattice_to_xy_hexagons(self):
        lattice_xy_coordinates = []
        for i in range(self.project.lattice_height): # height or y-dimension
            lattice_xy_coordinates_row = []
            for j in range(self.project.lattice_width): # width or x-dimension
                if j%2 == 1: x = (self.lattice_spacing*i) + self.lattice_spacing
                elif j%2 == 0: x = (self.lattice_spacing*i) + (self.lattice_spacing/2)
                y = (self.lattice_spacing*j) + (self.lattice_spacing/2)
                lattice_xy_coordinates_column = (x, y)
                lattice_xy_coordinates_row.append(lattice_xy_coordinates_column)
            lattice_xy_coordinates.append(lattice_xy_coordinates_row)
        return lattice_xy_coordinates



import tkinter as tk


class LayerConfigurator(tk.Frame):
    """
    This defines a window for setting properties of a single layer.
    """
    def __init__(self, layername, layerdepth, layeraction, task="create", master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        self.layeraction = layeraction
        self.layeraction.set(None)
        self.task = task
        
        self.lbl = tk.Label(self, text="Name")
        self.lbl.grid(row=0, column=0, padx=2, pady=4)
        self.name_entry = tk.Entry(self, textvariable=layername)
        self.name_entry.grid(row=0, column=1, padx=2, pady=4)

        self.depthvar = layerdepth
        self.lbl2 = tk.Label(self, text="z position " + u"(\u212B)")
        self.lbl2.grid(row=1, column=0, padx=2, pady=4)
        self.name_entry = tk.Entry(self, textvariable=self.depthvar)
        self.name_entry.grid(row=1, column=1, padx=2, pady=4)
        
        self.lbl3 = tk.Label(self)
        self.lbl3.grid(row=2, column=0, columnspan=2, padx=2, pady=4)

        self.cancel_button = tk.Button(self, text="Cancel", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="OK", command=self.ok_action)
        self.cancel_button.grid(row=3, column=0, padx=2, pady=4)
        self.ok_button.grid(row=3, column=1, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)
        self.master.bind('<Destroy>', self.destroy_event)

    def cancel_action(self):
        self.layeraction.set("cancel")
        self.master.destroy()

    def ok_action(self):
        try:
            int(self.depthvar.get())
        except ValueError:
            self.lbl3["text"] = "The z position must be a number"
            return
        self.layeraction.set(self.task)
        self.master.destroy()

    def destroy_event(self, event):
        self.layeraction.set("cancel")

    def return_event(self, event):
        self.ok_button.invoke()
        


import json
import random
import numpy as np
import MDAnalysis as mda


class ZLayer():
    def __init__(self):
        self.lattice = None
        self.name = None
        self.zdepth = 0


class Molecule():
    def __init__(self):
        self.path = None
        self.name = "Empty"
        self.color = "#FFFFFF"
        self.index = 0
        self.flipbool = 0
        self.rotatebool = 0

    def get_paint_props(self):
        """
        Gives a tuple of properties needed to paint this molecule: ID and color.
        """
        return (self.index, self.color)


class Blender():
    def __init__(self):
        self.name = "Blender"
        self.index = -1
        self.molecules = {}

    def get_paint_props(self):
        """
        Gives a tuple of properties needed to paint a single molecule from this 
        blend: ID and color. The molecule to be painted is given by the distribution function.
        """
        mol = self.distribute_next()
        return (mol.index, mol.color)

    def distribute_next(self):
        """
        Distribution function that picks one molecule from this blend
        """
        return random.choices(population=list(self.molecules.keys()), weights=list(self.molecules.values()))[0]


class SolventLayer():
    def __init__(self):
        self.name = "Solvent layer"
        self.lattice_spacing = 4.0
        self.start_offset = 0
        self.end_offset = -10
        self.molecules = {}


class Project():
    def __init__(self):
        pass

    def init_defaults(self):
        """
        Set everything to default values for a new project
        """
        self.lattice_width = 25
        self.lattice_height = 25
        self.lattice_spacing = 8
        self.lattice_major_gridlines = 5
        self.layer_count = 1
        self.molecule_count = 0
        self.solvent_molecule_count = 0
        self.solvent_layer_count = 0
        self.blender_count = 0
        self.blender_offset = 1000
        self.import_solute = None
        self.solute_buffer_space = 3
        self.solute_z = None

        self.layers = []
        self.molecules = []
        self.blenders = []

        self.solvent_molecules = []
        self.solvent_layers = []

    def edit_lattice_params(self, width, height, spacing, lines):
        """
        Change the lattice size of this project. The layers must resize to match.
        """
        self.lattice_spacing = spacing
        self.lattice_width = width
        self.lattice_height = height
        self.lattice_major_gridlines = lines

        for layer in self.layers:
            new_lattice = np.zeros((self.lattice_height, self.lattice_width))
            for i in range(np.amin([layer.lattice.shape[0], new_lattice.shape[0]])):
                for j in  range(np.amin([layer.lattice.shape[1], new_lattice.shape[1]])):
                    new_lattice[i,j] = layer.lattice[i,j]
            layer.lattice = new_lattice

    def new_layer(self):
        """
        Come up with starting parameters for a potential new layer
        A name and Z depth may already be provided
        """
        layer = ZLayer()
        layer.lattice = np.zeros((self.lattice_height, self.lattice_width), dtype='int')
        layer.name = "Layer {}".format(self.layer_count)
        layer.zdepth = 0

        return layer

    def add_layer(self, layer):
        """
        Add this Z layer to the project
        """
        self.layer_count += 1
        self.layers.append(layer)

    def delete_layer(self, layer):
        """
        Delete this layer from the project
        """
        self.layers.remove(layer)
        layer = None

    def add_molecule(self, molecule):
        """
        Add this molecule to the project
        """
        self.molecules.append(molecule)
        self.molecule_count += 1

    def add_solvent_molecule(self, molecule):
        """
        Add this as a solvent molecule to the project
        """
        self.solvent_molecules.append(molecule)
        self.solvent_molecule_count += 1

    def delete_molecule(self, molecule):
        """
        Delete this molecule from the project
        """
        self.molecules.remove(molecule)
        molecule = None

    def delete_solvent_molecule(self, molecule):
        """
        Delete this solvent molecule from the project
        """
        self.solvent_molecules.remove(molecule)
        molecule = None

    def new_molecule(self):
        """
        Come up with starting parameters for a potential new molecule
        """
        default_colors = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']
        molecule = Molecule()
        molecule.index = self.molecule_count
        molecule.name = "Mol {}".format(self.molecule_count + self.solvent_molecule_count)
        molecule.color = default_colors[(self.molecule_count - 1) % 10]
        return molecule

    def new_solvent_molecule(self):
        """
        Come up with starting parameters for a potential new solvent molecule
        """
        default_colors = ['#8175aa', '#6fb899', '#31a1b3', '#ccb22b', '#a39fc9', '#94d0c0', '#959c9e', '#027b8e', '#9f8f12', '#767676']
        molecule = Molecule()
        molecule.index = self.solvent_molecule_count
        molecule.name = "Mol {}".format(self.molecule_count + self.solvent_molecule_count)
        molecule.color = default_colors[(self.solvent_molecule_count) % 10]
        return molecule

    def project_loaded(self):
        """
        Makes sure the counts match what was in the previous project
        """
        for mol in self.molecules:
            if mol.index >= self.molecule_count:
                self.molecule_count = mol.index + 1
        for mol in self.solvent_molecules:
            if mol.index >= self.solvent_molecule_count:
                self.solvent_molecule_count = mol.index + 1

    def new_blender(self):
        """
        Create an empty blender. 
        To make it 1-indexed, the default name is blender_count + 1, since
        there is no "default" or "empty" blender on a new project.
        """
        blender = Blender()
        blender.name = "Blend {}".format(self.blender_count + 1)
        blender.index = self.blender_count + self.blender_offset
        return blender

    def add_blender(self, blender):
        """
        Add this blender to the project
        """
        self.blenders.append(blender)
        self.blender_count += 1

    def delete_blender(self, blender):
        """
        Delete this blender from the project
        """
        self.blenders.remove(blender)
        blender = None

    def new_solvent_layer(self):
        """
        Come up with starting parameters for a new solvent layer
        """
        layer = SolventLayer()
        layer.name = "Solvent layer {}".format(self.solvent_layer_count + 1)
        return layer

    def add_solvent_layer(self, layer):
        """
        Add this solvent layer to the project
        """
        self.solvent_layers.append(layer)
        self.solvent_layer_count += 1

    def delete_solvent_layer(self, layer):
        """
        Remove this solvent layer from the project
        """
        self.solvent_layers.remove(layer)
        layer = None

    def edit_solute_settings(self, file, bufspace, center):
        """
        Apply solute configuration so it can be loaded
        """
        self.import_solute = file
        self.solute_buffer_space = float(bufspace)
        if center is not None:
            self.solute_z = float(center)
        else:
            self.solute_z = None

    def load_solute(self, should_expand):
        """
        Import a solute into the project. Expands the lattice if needed.
        """
        if self.import_solute is not None:
            self.solute = mda.Universe(self.import_solute)
        else:
            return

        if self.solute_z is not None:
            positions = self.solute.atoms.positions
            solute_indices = list(set(list(np.where(positions[:,2] > self.solute_z-self.lattice_spacing)[0])) & set(list(np.where(positions[:,2] < self.solute_z+self.lattice_spacing)[0])))
            solute_center = np.array([np.mean(positions[:,0]), np.mean(positions[:,1]), np.mean(positions[:,2])])
            lattice_center = np.array([(self.lattice_spacing*self.lattice_width)/2., (self.lattice_spacing*self.lattice_height)/2., self.solute_z])
            self.solute.atoms.positions = positions + (lattice_center - solute_center)
            pass

        if should_expand:
            xmax = np.amax(self.solute.atoms.positions[:,0])
            ymax = np.amax(self.solute.atoms.positions[:,1])
            newwidth = self.lattice_width
            newheight = self.lattice_height
            if xmax > (self.lattice_width*self.lattice_spacing):
                newwidth = int((xmax + self.solute_buffer_space + self.lattice_spacing) // self.lattice_spacing)
            if ymax > (self.lattice_height*self.lattice_spacing):
                newheight = int((ymax + self.solute_buffer_space + self.lattice_spacing) // self.lattice_spacing)
            self.edit_lattice_params(newwidth, newheight, self.lattice_spacing, self.lattice_major_gridlines)

        for layer in self.layers:
            self.overlay_solute(layer)

    def overlay_solute(self, layer):
        """
        Overlay the imported solute onto one layer so that the lattice regions
        occupied by the solute become obstructed.
        """
        positions = self.solute.atoms.positions
        solute_indices = list(set(list(np.where(positions[:,2] > layer.zdepth-self.lattice_spacing)[0])) & set(list(np.where(positions[:,2] < layer.zdepth+self.lattice_spacing)[0])))
        self.remove_overlay(layer)
        for ind in solute_indices:
            x = positions[ind][0]
            y = positions[ind][1]
            n_pos_x = int((x+self.solute_buffer_space) // self.lattice_spacing)
            n_neg_x = int((x-self.solute_buffer_space) // self.lattice_spacing)
            n_pos_y = int((y+self.solute_buffer_space) // self.lattice_spacing)
            n_neg_y = int((y-self.solute_buffer_space) // self.lattice_spacing)
            n_range_x = list(range(n_neg_x, n_pos_x+1))
            n_range_y = list(range(n_neg_y, n_pos_y+1))
            for row in n_range_y:
                for col in n_range_x:
                    if (row >= 0) and (row < layer.lattice.shape[0]) and (col >= 0) and (col < layer.lattice.shape[1]):
                        np.flip(layer.lattice.swapaxes(0,1), axis=1)[col][row] = -1

    def remove_overlay(self, layer):
        """
        Set obstructed lattice site IDs from -1 back to 0
        """
        for row in range(layer.lattice.shape[0]):
            for col in range(layer.lattice.shape[1]):
                if layer.lattice[row][col] == -1:
                    layer.lattice[row][col] = 0
    


import tkinter as tk
from tkinter import filedialog


class SoluteImporter(tk.Frame):
    """
    This defines a window for importing a PDB file that will be merged with the project.
    """
    def __init__(self, project, file, master=None):
        super().__init__(master)
        self.master = master
        self.project = project
        self.grid(column=0, row=0)
        self.filevar = file
        self.bufvar = tk.StringVar()
        self.bufvar.set(self.project.solute_buffer_space)

        self.lblf = tk.Label(self, text="Path")
        self.lblf.grid(row=0, column=0, padx=2, pady=4)
        self.file_entry = tk.Entry(self, textvariable=self.filevar)
        self.file_entry.grid(row=0, column=1, columnspan=3, padx=2, pady=4, sticky=("E", "W"))
        self.file_button = tk.Button(self, text="Browse", command=self.file_select_action)
        self.file_button.grid(row=0, column=4, padx=2, pady=4)

        self.lblb = tk.Label(self, text="Buffer space " + u"(\u212B)")
        self.lblb.grid(row=1, column=1, columnspan=2, padx=2, pady=4)
        self.buf_entry = tk.Entry(self, textvariable=self.bufvar)
        self.buf_entry.grid(row=1, column=3, padx=2, pady=4)

        self.centerbool = tk.BooleanVar()
        self.centervar = tk.StringVar()
        self.centervar.set("0")
        if self.project.solute_z is not None:
            self.centervar.set(str(self.project.solute_z))
            self.centerbool.set(True)
        self.center_checkbox = tk.Checkbutton(self, text="Center solute at z "+ u"(\u212B)", variable=self.centerbool)
        self.center_checkbox.grid(row=2, column=1, columnspan=2, padx=2, pady=4, sticky=("W"))
        self.center_entry = tk.Entry(self, textvariable=self.centervar)
        self.center_entry.grid(row=2, column=3, padx=2, pady=4)

        self.expandbool = tk.BooleanVar()
        self.expand_checkbox = tk.Checkbutton(self, text="Expand grid to fit solute", variable=self.expandbool)
        self.expand_checkbox.grid(row=3, column=1, columnspan=3, padx=2, pady=4, sticky=("W"))

        self.cancel_button = tk.Button(self, text="Cancel", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="OK", command=self.ok_action)
        self.cancel_button.grid(row=4, column=0, padx=2, pady=4)
        self.ok_button.grid(row=4, column=4, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)

    def file_select_action(self):
        self.filevar.set(filedialog.askopenfilename(filetypes = (("PDB file","*.pdb"),("all files","*.*"))))

    def cancel_action(self):
        self.master.destroy()

    def ok_action(self):
        # Apply settings before loading
        if self.centerbool.get():
            center = self.centervar.get()
        else:
            center = None
        self.project.edit_solute_settings(self.filevar.get(), self.bufvar.get(), center)
        # Try to load the solute into the project
        try:
            self.project.load_solute(self.expandbool.get())
        except Exception as e:
            tk.messagebox.showinfo(message=e.strerror)
            self.project.import_solute = None
            return
        self.master.master.drawarea.refresh_tabs()
        self.master.destroy()

    def return_event(self, event):
        self.ok_button.invoke()
        



import tkinter as tk


class GridConfigurator(tk.Frame):
    """
    This defines a window for setting dimensions of the project at a global level.
    """
    def __init__(self, gridwidth, gridheight, gridspacing, gridlines, gridaction, task="modify", master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        self.gridaction = gridaction
        self.gridaction.set(None)
        self.task = task
        
        self.widthvar = gridwidth
        self.lbl_width = tk.Label(self, text="Lattice width")
        self.lbl_width.grid(row=0, column=0, padx=2, pady=4)
        self.width_entry = tk.Entry(self, textvariable=self.widthvar)
        self.width_entry.grid(row=0, column=1, padx=2, pady=4)

        self.heightvar = gridheight
        self.lbl_height = tk.Label(self, text="Lattice height")
        self.lbl_height.grid(row=1, column=0, padx=2, pady=4)
        self.height_entry = tk.Entry(self, textvariable=self.heightvar)
        self.height_entry.grid(row=1, column=1, padx=2, pady=4)

        self.spacingvar = gridspacing
        self.lbl_area = tk.Label(self, text="Cell spacing " + u"\u212B")
        self.lbl_area.grid(row=2, column=0, padx=2, pady=4)
        self.area_entry = tk.Entry(self, textvariable=self.spacingvar)
        self.area_entry.grid(row=2, column=1, padx=2, pady=4)

        self.linesvar = gridlines
        self.lbl_lines = tk.Label(self, text="Major gridlines")
        self.lbl_lines.grid(row=3, column=0, padx=2, pady=4)
        self.lines_entry = tk.Entry(self, textvariable=self.linesvar)
        self.lines_entry.grid(row=3, column=1, padx=2, pady=4)

        self.lbl3 = tk.Label(self)
        self.lbl3.grid(row=4, column=0, columnspan=2, padx=2, pady=4)
        if self.master.master.project.import_solute is not None:
            self.lbl3["text"] = "Warning: a solute has been imported.\nChanging cell spacing may disrupt the painting."

        self.cancel_button = tk.Button(self, text="Cancel", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="OK", command=self.ok_action)
        self.cancel_button.grid(row=5, column=0, padx=2, pady=4)
        self.ok_button.grid(row=5, column=1, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)
        self.master.bind('<Destroy>', self.destroy_event)

    def cancel_action(self):
        self.gridaction.set("cancel")
        self.master.destroy()

    def ok_action(self):
        try:
            int(self.widthvar.get())
            int(self.heightvar.get())
            int(self.spacingvar.get())
        except ValueError:
            self.lbl3["text"] = "Only numeric values can be used"
            return
        self.gridaction.set(self.task)
        self.master.destroy()

    def destroy_event(self, event):
        self.gridaction.set("cancel")

    def return_event(self, event):
        self.ok_button.invoke()
        



import os
import shutil
import json

import tkinter as tk
from tkinter import filedialog


class ProjSaver(tk.Frame):
    """
    This defines a window for saving the project information to a text file.
    The goal is to be able to copy the workspace from one machine to another.
    For this reason, molecule source files can be copied, and relative paths can be used.
    """
    def __init__(self, project, savfile, copybool, master=None):
        super().__init__(master)
        self.master = master
        self.project = project
        self.grid(column=0, row=0)
        self.savefilevar = savfile
        self.copyfilesbool = copybool

        self.lblf = tk.Label(self, text="Path")
        self.lblf.grid(row=0, column=0, padx=2, pady=4)
        self.file_entry = tk.Entry(self, textvariable=self.savefilevar)
        self.file_entry.grid(row=0, column=1, columnspan=3, padx=2, pady=4, sticky=("E", "W"))
        self.file_button = tk.Button(self, text="Browse", command=self.file_select_action)
        self.file_button.grid(row=0, column=4, padx=2, pady=4)

        self.copy_checkbox = tk.Checkbutton(self, text="Copy molecule source files", variable=self.copyfilesbool)
        self.copy_checkbox.grid(row=1, column=1, columnspan=2, padx=2, pady=4, sticky=("W"))

        self.cancel_button = tk.Button(self, text="Cancel", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="Save", command=self.ok_action)
        self.cancel_button.grid(row=2, column=0, padx=2, pady=4)
        self.ok_button.grid(row=2, column=4, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)

    def file_select_action(self):
        self.savefilevar.set(filedialog.asksaveasfilename(initialdir = "./",title = "Select file",filetypes = (("JSON file","*.json"),("all files","*.*"))))

    def cancel_action(self):
        self.master.destroy()

    def ok_action(self):
        destdir = self.get_destdir()
        if not destdir:
            return
        if self.copyfilesbool.get():
            if not self.copy_molecules(destdir):
                return
        if self.copyfilesbool.get():
            if self.project.import_solute is not None:
                self.copy_solute(destdir)
        self.write_project_file()
        self.master.destroy()

    def return_event(self, event):
        self.ok_button.invoke()

    def copy_molecules(self, destdir):
        """
        Make copies of molecule source files in the same location as the saved
        project. This is for sharing the project data among multiple users or
        workstations. The project workspace will then be able to load molecule
        data from a known location, rather than ask the user to browse to the
        correct files for each one.
        """
        for mol in self.project.molecules:
            if mol.index == 0:
                continue
            success = self.validate_mol(mol)
            if not success:
                return False

        for mol in self.project.molecules:
            if mol.index == 0:
                continue
            self.copy_mol(mol, destdir)

        return True

    def copy_solute(self, destdir):
        """
        Make a copy of the solute file along with the other project files.
        Check if the file exists and prompt to update it if it does not.
        Modifies the path to only refer to the filename.
        """
        exists = True
        try:
            exists = os.path.exists(self.project.import_solute)
        except:
            exists = False
        while not exists:
            newpath = None
            result = tk.messagebox.askyesno(
                message="The source path for solute doesn't exist.\n{}\nDo you want to choose a new path?".format(self.project.import_solute))
            if result:
                newpath = filedialog.askopenfilename(filetypes = (("PDB file","*.pdb"),("all files","*.*")))
            else:
                self.project.import_solute = None
                return
            try:
                exists = os.path.exists(self.project.import_solute)
            except:
                exists = False
        try:
            shutil.copy2(self.project.import_solute, destdir)
            self.project.import_solute = os.path.basename(self.project.import_solute)
        except OSError as e:
            tk.messagebox.showinfo(message=e.strerror)

    def get_destdir(self):
        """
        Get the destination directory based on the savefile location. If the
        location is empty or not valid, returns None.
        """
        try:
            dir = os.path.dirname(self.savefilevar.get())
            return dir
        except:
            return None

    def validate_mol(self, molecule):
        """
        Check if this molecule has a valid source file. If yes, it is OK to 
        copy. If no, ask the user if they need to modify this molecule.
        """
        exists = self.mol_exists(molecule)
        
        if not exists:
            result = tk.messagebox.askyesno(
                message="The source path for {} doesn't exist. Do you want to modify this molecule?".format(molecule.name))
            if result:
                self.master.master.cmds.molresumeaction = self.ok_action
                self.master.master.cmds.newmolecule = molecule
                self.master.master.cmds.edit_molecule_action()
                return False
            else:
                return True
        else:
            return True

    def mol_exists(self, molecule):
        """
        Check if a molecule's source file exists
        """
        exists = True
        try:
            exists = os.path.exists(molecule.path)
        except:
            exists = False
        return exists

    def copy_mol(self, molecule, destdir):
        """
        Copy the molecule source file to the destination directory. If the 
        source file does not exist, suppress any errors and do not copy it.
        This is called after validation, which gives the user the opportunity
        to modify any molecules whose source file does not exist.

        This will also modify the molecule to refer to only the filename with 
        no path component.
        """
        if self.mol_exists(molecule):
            try:
                shutil.copy2(molecule.path, destdir)
                molecule.path = os.path.basename(molecule.path)
            except OSError as e:
                tk.messagebox.showinfo(message=e.strerror)

    def write_project_file(self):
        """
        Write a file corresponding to the current project's contents.
        TODO: modify the JSON exporter so that the layers' lattices output more
        than one element per line.
        """
        proj = {
            'settings' : {
                'lattice_width' : self.project.lattice_width,
                'lattice_height' : self.project.lattice_height,
                'lattice_spacing' : self.project.lattice_spacing,
                'lattice_major_gridlines' : self.project.lattice_major_gridlines,
                'import_solute' : self.project.import_solute,
                'solute_buffer_space' : self.project.solute_buffer_space,
                'solute_z' : self.project.solute_z
            },
            'molecules' : [],
            'blenders' : [],
            'layers' : [],
        }

        for mol in self.project.molecules:
            if mol.index == 0:
                continue
            proj['molecules'].append({
                'path' : mol.path,
                'name' : mol.name,
                'color' : mol.color,
                'index' : mol.index,
                'flip' : mol.flipbool,
                'rotate' : mol.rotatebool,
            })

        for blend in self.project.blenders:
            weights = {}
            for key, val in blend.molecules.items():
                weights[key.index] = val
            proj['blenders'].append({
                'name' : blend.name,
                'index' : blend.index,
                'weights' : weights,
            })

        for layer in self.project.layers:
            proj['layers'].append({
                'name' : layer.name,
                'zpos' : layer.zdepth,
                'lattice' : layer.lattice.tolist(),
            })
        
        try:
            with open(self.savefilevar.get(), 'w') as savefile:
                json.dump(proj, savefile, indent=4)
        except Exception as e:
            tk.messagebox.showinfo(message=str(e))



import os
import tkinter as tk
from tkinter import ttk

class Toolbox(tk.Frame):
    """
    This defines the painting tool buttons at the side of the window.
    """
    def __init__(self, master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)

        icondir = os.path.join(os.path.dirname(__file__), 'icons')

        self.pencil_icon = tk.PhotoImage(file=os.path.join(icondir, 'pencil.png'))
        self.rect_icon = tk.PhotoImage(file=os.path.join(icondir, 'square.png'))
        self.spray_icon = tk.PhotoImage(file=os.path.join(icondir, 'spraycan.png'))
        self.zoomin_icon = tk.PhotoImage(file=os.path.join(icondir, 'zoomin.png'))
        self.zoomout_icon = tk.PhotoImage(file=os.path.join(icondir, 'zoomout.png'))

        self.btn_zoomout = tk.Button(self, image=self.zoomout_icon, command=self.master.cmds.zoom_out_action, height=40, width=40)
        self.btn_zoomin = tk.Button(self, image=self.zoomin_icon, command=self.master.cmds.zoom_in_action, height=40, width=40)
        self.btn_pencil = tk.Button(self, image=self.pencil_icon, command=self.activate_pencil, height=40, width=40)
        self.btn_rect = tk.Button(self, image=self.rect_icon, command=self.activate_rect, height=40, width=40)
        self.btn_spray = tk.Button(self, image=self.spray_icon, command=self.activate_spray, height=40, width=40)

        self.spray_width_scale = tk.Scale(self, from_=1, to=6, orient="horizontal", label='Spray radius', length=130)
        
        self.btn_zoomout.grid(column=0, row=0, columnspan=1, rowspan=1, sticky=("W"))
        self.btn_zoomin.grid(column=1, row=0, columnspan=1, rowspan=1, sticky=("W"))
        self.btn_pencil.grid(column=0, row=1, columnspan=1, rowspan=1, sticky=("W"))
        self.btn_rect.grid(column=1, row=1, columnspan=1, rowspan=1, sticky=("W"))
        self.btn_spray.grid(column=2, row=1, columnspan=1, rowspan=1, sticky=("W"))
        self.spray_width_scale.grid(column=0, row=5, columnspan=3, rowspan=1, sticky=("N", "S", "W"))

    def activate_pencil(self):
        #self.btn_pencil.state(['pressed'])
        #self.btn_rect.state(['!pressed'])
        #self.btn_spray.state(['!pressed'])
        self.master.drawarea.set_tool('pencil')

    def activate_rect(self):
        #self.btn_pencil.state(['!pressed'])
        #self.btn_rect.state(['pressed'])
        #self.btn_spray.state(['!pressed'])
        self.master.drawarea.set_tool('rect')

    def activate_spray(self):
        #self.btn_pencil.state(['!pressed'])
        #self.btn_rect.state(['!pressed'])
        #self.btn_spray.state(['pressed'])
        self.master.drawarea.set_tool('spray')

    def open_noddle_popout(self):
        """
        Opens a popous window to the scale bar for the spray can
        """
        nozzle_popup = tk.Toplevel(self.gui)



import tkinter as tk
from tkinter import colorchooser, filedialog


class MoleculeConfigurator(tk.Frame):
    """
    This defines a window for setting properties of a molecule.
    """
    def __init__(self, name, color, file, flipbool, rotatebool, action, task="create", master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        self.action = action
        self.action.set(None)
        self.task = task
        self.filevar = file

        self.lblf = tk.Label(self, text="Path")
        self.lblf.grid(row=0, column=0, padx=2, pady=4)
        self.file_entry = tk.Entry(self, textvariable=self.filevar)
        self.file_entry.grid(row=0, column=1, columnspan=3, padx=2, pady=4, sticky=("E", "W"))
        self.file_button = tk.Button(self, text="Browse", command=self.file_select_action)
        self.file_button.grid(row=0, column=4, padx=2, pady=4)

        self.lbl = tk.Label(self, text="Name")
        self.lbl.grid(row=1, column=0, padx=2, pady=4)
        self.name_entry = tk.Entry(self, textvariable=name)
        self.name_entry.grid(row=1, column=1, columnspan=3, padx=2, pady=4, sticky=("E", "W"))

        self.colorvar = color
        self.lblc = tk.Label(self, text="Color")
        self.lblc.grid(row=2, column=0, padx=2, pady=4)
        self.color_entry = tk.Entry(self, textvariable=self.colorvar)
        self.canvas = tk.Canvas(self, width=20, height=20, background=self.colorvar.get())
        self.canvas.grid(row=2, column=1, padx=2, pady=4)
        self.color_entry.grid(row=2, column=2, columnspan=2, padx=2, pady=4, sticky=("E", "W"))
        self.color_button = tk.Button(self, text="Select", command=self.color_select_action)
        self.color_button.grid(row=2, column=4, padx=2, pady=4)

        self.flipbool = flipbool
        self.flip_checkbox = tk.Checkbutton(self, text="Flip 180"+u"\N{DEGREE SIGN}"+" over x-axis", variable=self.flipbool)
        self.flip_checkbox.grid(row=3, column=2, columnspan=2, padx=2, pady=4, sticky=("W"))
        self.rotatebool = rotatebool
        self.rotate_checkbox = tk.Checkbutton(self, text="Randomly rotate around z-axis", variable=self.rotatebool)
        self.rotate_checkbox.grid(row=4, column=2, columnspan=2, padx=2, pady=4, sticky=("W"))

        self.cancel_button = tk.Button(self, text="Cancel", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="OK", command=self.ok_action)
        self.cancel_button.grid(row=5, column=0, padx=2, pady=4)
        self.ok_button.grid(row=5, column=4, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)
        self.master.bind('<Destroy>', self.destroy_event)

    def file_select_action(self):
        self.filevar.set(filedialog.askopenfilename())

    def color_select_action(self):
        newcolor = self.colorvar.get()
        try:
            newcolor = colorchooser.askcolor(initialcolor=newcolor)
        except Exception:
            newcolor = colorchooser.askcolor()

        if newcolor[1] is not None:
            self.colorvar.set(newcolor[1])
            self.canvas["background"] = newcolor[1]

    def cancel_action(self):
        self.action.set("cancel")
        self.master.destroy()

    def ok_action(self):
        self.action.set(self.task)
        self.master.destroy()

    def destroy_event(self, event):
        self.action.set("cancel")

    def return_event(self, event):
        self.ok_button.invoke()
        



import random
import json
import tkinter as tk
from tkinter import colorchooser, filedialog


class BlendConfigurator(tk.Frame):
    """
    This defines a window for defining a blend of molecules.
    """
    def __init__(self, name, vals, action, task="create", master=None):
        super().__init__(master)
        self.master = master
        self.grid(column=0, row=0)
        self.action = action
        self.action.set(None)
        self.task = task
        self.molecules = []
        self.valsvar = vals
        self.resultdict = {}
        self.set_vals_from_string(self.valsvar.get())

        self.preview = tk.Canvas(self, width=20, height=20, background="#FFFFFF")
        self.preview.grid(row=0, column=0, padx=2, pady=4)
        self.preview_image = tk.PhotoImage(master=self, width=20, height=20)
        self.preview.create_image("0", "0", anchor="nw", image=self.preview_image)
        self.lbl = tk.Label(self, text="Name")
        self.lbl.grid(row=0, column=1, padx=2, pady=4)
        self.name_entry = tk.Entry(self, textvariable=name)
        self.name_entry.grid(row=0, column=2, columnspan=2, padx=2, pady=4)

        self.canvas = tk.Canvas(self, borderwidth=0)
        self.canvas.grid(row=1, column=0, columnspan=3, padx=4, pady=1, sticky=("E", "S", "W", "N"))
        self.mlist = tk.Frame(self.canvas)
        self.mlist.grid(column=0, row=0)
        self.molheader = tk.Label(self.mlist, text="Molecule")
        self.weightheader = tk.Label(self.mlist, text="Weight")
        self.molheader.grid(row=0, column=1)
        self.weightheader.grid(row=0, column=2)

        self.lbl3 = tk.Label(self)
        self.lbl3.grid(row=2, column=0, columnspan=4, padx=2, pady=4)

        self.cancel_button = tk.Button(self, text="Cancel", command=self.cancel_action)
        self.ok_button = tk.Button(self, text="OK", command=self.ok_action)
        self.cancel_button.grid(row=3, column=0, padx=2, pady=4)
        self.ok_button.grid(row=3, column=4, padx=2, pady=4)

        self.master.bind('<Return>', self.return_event)
        self.master.bind('<Destroy>', self.destroy_event)
        self.list_molecules()
        self.draw_preview()

    def list_molecules(self):
        for mol in self.master.master.project.molecules:
            var=tk.StringVar()
            var.set(str(self.resultdict.get(mol.index,"0")))
            self.molecules.append({
                "color": tk.Canvas(self.mlist, width=20, height=20, background=mol.color),
                "label": tk.Label(self.mlist, text=str(mol.name)),
                "weight": var,
                "entry": tk.Entry(self.mlist, textvariable=var, validate="none", validatecommand=self.evaluate),
                "molecule": mol
            })
        self.display_molecules()

    def display_molecules(self):
        for i in range(len(self.molecules)):
            self.molecules[i]["color"].grid(row=i+1, column=0, padx=0, pady=4, sticky="W")
            self.molecules[i]["label"].grid(row=i+1, column=1, padx=0, pady=4, sticky="W")
            self.molecules[i]["entry"].grid(row=i+1, column=2, padx=0, pady=4, sticky="E")
            self.molecules[i]["entry"]["validate"] = "all"

    def evaluate(self):
        self.resultdict = {}
        for mol in self.molecules:
            try:
                index = mol["molecule"].index
                weight = int(mol["weight"].get())
                if weight > 0:
                    self.resultdict[index] = weight
            except ValueError as e:
                self.resultdict = {}
                return 1
        if not self.resultdict:
            self.lbl3["text"] = "The weights must be more than zero"
        else:
            self.lbl3["text"] = ""
        self.draw_preview()
        return 1

    def draw_preview(self):
        for row in range(int(self.preview_image["height"])):
            for col in range(int(self.preview_image["width"])):
                self.preview_image.put(self.get_color(), to=(row, col))

    def get_color(self):
        if not self.resultdict:
            return "#FFFFFF"
        choice = random.choices(population=list(self.resultdict.keys()), weights=list(self.resultdict.values()))[0]
        for mol in self.molecules:
            if mol["molecule"].index == choice:
                return mol["molecule"].color
        return "#FFFFFF"

    def set_vals_from_string(self, valstr):
        try:
            srcdic = json.loads(valstr)
        except json.decoder.JSONDecodeError:
            return
        for key, val in srcdic.items():
            self.resultdict[int(key)] = val

    def cancel_action(self):
        self.action.set("cancel")
        self.master.destroy()

    def ok_action(self):
        self.evaluate()
        if not self.resultdict:
            self.lbl3["text"] = "The weights must be positive integers"
            return
        self.valsvar.set(json.dumps(self.resultdict))
        self.action.set(self.task)
        self.master.destroy()

    def destroy_event(self, event):
        self.action.set("cancel")

    def return_event(self, event):
        self.ok_button.invoke()
        



import setuptools

setuptools.setup(
    name="MolPainter",
    version="1.1.2",
    description="Tool for drawing complex planar molecular systems of arbitrary composition and molecule placement",
    long_description=open('README.md', 'r').read(),
    long_description_content_type="text/markdown",
    url='https://github.com/gpantel/MolPainter',
    author='George Pantelopulos, Aaron Liberatore',
    license='MIT',
    classifiers=[
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Chemistry',
    ],
    packages=["MolPainter", "MolSolvator"],
    package_data={
        "": ['icons/*.png'],
    },
    install_requires=['numpy', 'scipy', 'toml', 'MDAnalysis'],
    entry_points={
        'console_scripts': [
            'molpainter=MolPainter.painter:main',
            'molsolvator=MolSolvator.MolSolvator:main',
        ],
    },
)



import json
import os
import tkinter as tk
import MDAnalysis as mda
import numpy as np

from .project import Molecule, Blender, ZLayer
from .layer_configurator import LayerConfigurator
from .molecule_configurator import MoleculeConfigurator
from .blend_configurator import BlendConfigurator
from .grid_configurator import GridConfigurator
from .proj_saver import ProjSaver
from .exporter import Exporter
from .solute_importer import SoluteImporter


class Commands:
    """
    This defines the actions that happen when buttons are clicked.
    """
    def __init__(self, gui, project):
        self.gui=gui
        self.project=project

        self.layername = tk.StringVar()
        self.layerdepth = tk.StringVar()
        self.layeraction = tk.StringVar()
        self.layeraction.trace_variable("w", self.layer_modified)
        self.layer_settings = None

        self.molname = tk.StringVar()
        self.molcolor = tk.StringVar()
        self.molpath = tk.StringVar()
        self.molflipbool = tk.IntVar()
        self.molrotatebool = tk.IntVar()
        self.molaction = tk.StringVar()
        self.molaction.trace_variable("w", self.molecule_modified)
        self.molecule_settings = None
        self.molresumeaction = None

        self.blendname = tk.StringVar()
        self.blendvals = tk.StringVar()
        self.blendaction = tk.StringVar()
        self.blendaction.trace_variable("w", self.blend_modified)
        self.blend_settings = None

        self.gridwidth = tk.StringVar()
        self.gridheight = tk.StringVar()
        self.gridspacing = tk.StringVar()
        self.gridaction = tk.StringVar()
        self.gridlines = tk.StringVar()
        self.gridaction.trace_variable("w", self.grid_modified)
        self.grid_settings = None

        self.exportfilevar = tk.StringVar()
        self.exportfilevar.set("output.pdb")
        self.importfilevar = tk.StringVar()

        self.projfilevar = tk.StringVar()
        self.projfilevar.set(os.path.join(os.path.expanduser('~'), "project.json"))
        self.copyfilesbool = tk.BooleanVar()
        self.copyfilesbool.set(False)


    def new_project_action(self):
        """
        Command to start a new project. Clears the existing settings and 
        creates a default project with one empty layer.
        """
        self.project.init_defaults()
        self.gui.mlist.molecules = []
        self.gui.mlist.blends = []
        self.gui.drawarea.layers = []
        self.gui.drawarea.forget_all_layers()
        newlayer = self.project.new_layer()
        self.gui.layer_created(newlayer)
        self.project.add_layer(newlayer)
        empty_molecule = Molecule()
        self.project.add_molecule(empty_molecule)
        self.gui.molecule_created(empty_molecule)
        self.gui.molecule_selected(empty_molecule)

    def save_project_action(self):
        """
        Command to save the working data for later use. Does not export the 
        actual molecules, for which 'export' should be used instead.
        """
        save_popup = tk.Toplevel(self.gui)
        ProjSaver(self.project, self.projfilevar, self.copyfilesbool, save_popup)

    def load_project_action(self):
        """
        Command to load project data. Attempts to validate molecule paths and
        prompts for update if a path is missing.
        """
        self.projfilevar.set(tk.filedialog.askopenfilename(initialdir = os.path.dirname(self.projfilevar.get()), title = "Select file",filetypes = (("JSON file","*.json"),("all files","*.*"))))
        path = self.projfilevar.get()
        if not self.file_exists(path):
            return
        with open(path, "r") as projfile:
            proj = json.load(projfile)
            self.project.init_defaults()
            self.gui.mlist.molecules = []
            self.gui.mlist.blends = []
            self.gui.drawarea.layers = []
            self.gui.drawarea.forget_all_layers()
            empty_molecule = Molecule()
            self.project.add_molecule(empty_molecule)
            self.gui.molecule_created(empty_molecule)
            self.project.edit_lattice_params(
                proj['settings']['lattice_width'],
                proj['settings']['lattice_height'],
                proj['settings']['lattice_spacing'],
                proj['settings']['lattice_major_gridlines']
                )
            for mol in proj['molecules']:
                newmol = Molecule()
                newmol.name = mol['name']
                newmol.color = mol['color']
                newmol.index = mol['index']
                newmol.flipbool = mol['flip']
                newmol.rotatebool = mol['rotate']
                newmol.path = mol['path']
                if newmol.path and not self.file_exists(newmol.path):
                    # Validation of paths - first check if a copy is included
                    # in the project location, then prompt the user to select
                    # a file if it could not be determined.
                    fname = os.path.basename(newmol.path)
                    copied_file = os.path.join(os.path.dirname(path), fname)
                    if not self.file_exists(copied_file):
                        confirm = tk.messagebox.askyesno(message="The path '{}' does not exist for molecule '{}'. Do you want to select a new path for '{}'?".format(newmol.path, newmol.name, newmol.name))
                        if confirm:
                            newmol.path = tk.filedialog.askopenfilename(initialdir = os.path.dirname(path), title = "Select file",filetypes = (("PDB file","*.pdb"),("all files","*.*")))
                self.project.add_molecule(newmol)
                self.gui.molecule_created(newmol)
            for blend in proj['blenders']:
                newblend = Blender()
                newblend.name = blend['name']
                newblend.index = blend['index']
                for key, val in blend['weights'].items():
                    for mol in self.project.molecules:
                        if mol.index == int(key):
                            newblend.molecules[mol] = val
                self.gui.blend_created(newblend)
                self.project.add_blender(newblend)
            for layer in proj['layers']:
                newlayer = ZLayer()
                newlayer.name = layer['name']
                newlayer.zdepth = layer['zpos']
                newlayer.lattice = np.array(layer['lattice'])
                self.gui.layer_created(newlayer)
                self.project.add_layer(newlayer)
            # New solute import feature. Suppress errors for backward compatibility.
            try:
                if proj['settings']['import_solute']:
                    solute_path = proj['settings']['import_solute']
                    if not self.file_exists(solute_path):
                        solute_path = os.path.join(os.path.dirname(path), os.path.basename(solute_path))
                    self.project.edit_solute_settings(solute_path, proj['settings']['solute_buffer_space'], proj['settings']['solute_z'])
                    try:
                        self.project.load_solute(False)
                    except Exception as e:
                        tk.messagebox.showinfo(message=e.strerror)
                        self.project.import_solute = None
            except KeyError:
                pass
            self.gui.drawarea.refresh_tabs()
            self.project.project_loaded()

    def file_exists(self, path):
        """
        Check if a file exists
        """
        exists = True
        try:
            exists = os.path.exists(path)
        except:
            exists = False
        return exists

    def grid_setup_action(self):
        """
        Command to change grid size and lattice spacing. Uses a configurator to
        collect values 
        """
        if self.grid_settings is not None:
            return
        self.gridwidth.set(self.project.lattice_width)
        self.gridheight.set(self.project.lattice_height)
        self.gridspacing.set(self.project.lattice_spacing)
        self.gridlines.set(self.project.lattice_major_gridlines)
        grid_popup = tk.Toplevel(self.gui)
        self.grid_settings = GridConfigurator(self.gridwidth, self.gridheight, self.gridspacing, self.gridlines, self.gridaction, "modify", grid_popup)

    def grid_modified(self, arg1, arg2, arg3):
        """
        Determine if the grid should be modified with the new settings or if the
        changes should be discarded.
        """
        if self.gridaction.get() == "modify":
            self.project.edit_lattice_params(int(self.gridwidth.get()), int(self.gridheight.get()), int(self.gridspacing.get()), int(self.gridlines.get()))
            if self.project.import_solute is not None:
                for layer in self.project.layers:
                    self.project.overlay_solute(layer)
            self.grid_settings = None
            self.gui.drawarea.refresh_tabs()
        elif self.gridaction.get() == "cancel":
            self.grid_settings = None

    def new_layer_action(self):
        """
        Command to create a new layer. Sets up a layer with default values,
        then hands it off to the configurator to be modified. If the layer
        was accepted, notify the GUI and add it to the project. If it was
        rejected, delete the layer.
        """
        if self.layer_settings is not None:
            return
        self.newlayer = self.project.new_layer()
        self.layername.set(self.newlayer.name)
        self.layerdepth.set(str(self.newlayer.zdepth))
        layer_popup = tk.Toplevel(self.gui)
        self.layer_settings = LayerConfigurator(self.layername, self.layerdepth, self.layeraction, "create", layer_popup)
    
    def edit_layer_action(self):
        """
        Command to edit a layer. Set self.newlayer prior to calling. This takes
        the layer and hands it off to the configurator to be modified.
        If the config was accepted, modify the layer in the project.
        """
        if self.layer_settings is not None:
            return
        self.layername.set(self.newlayer.name)
        self.layerdepth.set(str(self.newlayer.zdepth))
        layer_popup = tk.Toplevel(self.gui)
        self.layer_settings = LayerConfigurator(self.layername, self.layerdepth, self.layeraction, "modify", layer_popup)

    def delete_layer_action(self):
        """
        Command to delete a layer. Set self.newlayer before calling. Asks for 
        confirmation before deleting.
        """
        if self.layer_settings is not None:
            return
        confirm = tk.messagebox.askyesno(message='Really delete layer "{}"?'.format(self.newlayer.name))
        if confirm:
            self.gui.layer_removed(self.newlayer)
            self.project.delete_layer(self.newlayer)
        
        self.layer_settings = None
        self.newlayer = None

    def layer_modified(self, arg1, arg2, arg3):
        """
        Determine if a layer should be modified with the new settings or if the
        changes should be discarded.
        """
        if self.layeraction.get() == "create":
            self.newlayer.name = self.layername.get()
            self.newlayer.zdepth = int(self.layerdepth.get())
            self.project.add_layer(self.newlayer)
            self.gui.layer_created(self.newlayer)
            self.layer_settings = None
            self.newlayer = None
        elif self.layeraction.get() == "modify":
            self.newlayer.name = self.layername.get()
            self.newlayer.zdepth = int(self.layerdepth.get())
            self.gui.layer_modified(self.newlayer)
            self.layer_settings = None
            self.newlayer = None
        elif self.layeraction.get() == "cancel":
            self.layer_settings = None
            self.newlayer = None

    def new_molecule_action(self):
        """
        Command to create a new molecule type. Creates a template with default
        values, then hands it off to the configurator to be modified. If it was
        accepted, notify the GUI and add it to the project. If rejected, 
        simply delete it.
        """
        if self.molecule_settings is not None:
            return
        self.newmolecule = self.project.new_molecule()
        self.molname.set(self.newmolecule.name)
        self.molcolor.set(self.newmolecule.color)
        molecule_popup = tk.Toplevel(self.gui)
        self.molecule_settings = MoleculeConfigurator(self.molname, self.molcolor, self.molpath, self.molflipbool, self.molrotatebool, self.molaction, "create", molecule_popup)

    def copy_molecule_action(self):
        """
        Command to copy a molecule. Takes all properties of the source molecule,
        creates a new molecule with those properties, and opens the configurator
        for further modification.
        """
        if self.molecule_settings is not None:
            return
        self.molname.set(self.newmolecule.name + " copy")
        newcolorval = self.newmolecule.color.lstrip('#')
        red = int(newcolorval[0:2], base=16)
        green = int(newcolorval[2:4], base=16)
        blue = int(newcolorval[4:6], base=16)
        red = int((red + 0xFF) / 2)
        green = int((green + 0xFF) / 2)
        blue = int((blue + 0xFF) / 2)
        self.molcolor.set('#{:02x}{:02x}{:02x}'.format(red, green, blue))
        self.molpath.set(self.newmolecule.path)
        self.molflipbool.set(self.newmolecule.flipbool)
        self.molrotatebool.set(self.newmolecule.rotatebool)
        molecule_popup = tk.Toplevel(self.gui)
        self.newmolecule = self.project.new_molecule()
        self.molecule_settings = MoleculeConfigurator(self.molname, self.molcolor, self.molpath, self.molflipbool, self.molrotatebool, self.molaction, "create", molecule_popup)

    def edit_molecule_action(self):
        """
        Command to edit a molecule. Set self.newmolecule prior to calling. This
        takes the molecule and hands it off to the configurator to be modified.
        If the config was accepted, modify the molecule in the project.
        """
        if self.molecule_settings is not None:
            return
        self.molname.set(self.newmolecule.name)
        self.molcolor.set(self.newmolecule.color)
        self.molpath.set(self.newmolecule.path)
        self.molflipbool.set(self.newmolecule.flipbool)
        self.molrotatebool.set(self.newmolecule.rotatebool)
        molecule_popup = tk.Toplevel(self.gui)
        self.molecule_settings = MoleculeConfigurator(self.molname, self.molcolor, self.molpath, self.molflipbool, self.molrotatebool, self.molaction, "modify", molecule_popup)

    def delete_molecule_action(self):
        """
        Command to delete a molecule. Set self.newmolecule before calling. Asks
        for confirmation before deleting.
        """
        if self.molecule_settings is not None:
            return
        confirm = tk.messagebox.askyesno(message='Really delete molecule "{}"?'.format(self.newmolecule.name))
        if confirm:
            self.gui.molecule_removed(self.newmolecule)
            self.project.delete_molecule(self.newmolecule)
            self.gui.molecule_modified()
        
        self.molecule_settings = None
        self.newmolecule = None

    def molecule_resume_action(self):
        """
        Resume an operation that was interrupted because a molecule needed to
        be modified.
        """
        if self.molresumeaction:
            self.molresumeaction()
            self.molresumeaction = None

    def molecule_modified(self, arg1, arg2, arg3):
        """
        Determines if a molecule should be modified with the new settings or if
        the changes should be discarded.
        """
        if self.molaction.get() == "create":
            self.newmolecule.name = self.molname.get()
            self.newmolecule.path = self.molpath.get()
            self.newmolecule.color = self.molcolor.get()
            self.newmolecule.flipbool = self.molflipbool.get()
            self.newmolecule.rotatebool = self.molrotatebool.get()
            self.gui.molecule_created(self.newmolecule)
            self.project.add_molecule(self.newmolecule)
            self.molecule_settings = None
            self.newmolecule = None
        elif self.molaction.get() == "modify":
            self.newmolecule.name = self.molname.get()
            self.newmolecule.path = self.molpath.get()
            self.newmolecule.color = self.molcolor.get()
            self.newmolecule.flipbool = self.molflipbool.get()
            self.newmolecule.rotatebool = self.molrotatebool.get()
            self.gui.molecule_modified()
            self.molecule_settings = None
            self.newmolecule = None
            self.molecule_resume_action()
        elif self.molaction.get() == "cancel":
            self.molecule_settings = None
            self.newmolecule = None
            self.molecule_resume_action()

    def new_blend_action(self):
        """
        Command to create a new blend of molecules. Creates a template with default
        values, then hands it off to the configurator to be modified. If it was
        accepted, notify the GUI and add it to the project. If rejected, 
        simply delete it.
        """
        if self.blend_settings is not None:
            return
        self.newblend = self.project.new_blender()
        self.blendname.set(self.newblend.name)
        self.blendvals.set("")
        blend_popup = tk.Toplevel(self.gui)
        self.blend_settings = BlendConfigurator(self.blendname, self.blendvals, self.blendaction, "create", blend_popup)

    def copy_blend_action(self):
        """
        Command to copy a blend definition. Takes the definition of the source blend
        and passes it to the configurator to be modified.
        """
        if self.blend_settings is not None:
            return
        self.blendname.set(self.newblend.name + " copy")
        self.blendvals.set(self.get_val_string_from_blend(self.newblend))
        self.newblend = self.project.new_blender()
        blend_popup = tk.Toplevel(self.gui)
        self.blend_settings = BlendConfigurator(self.blendname, self.blendvals, self.blendaction, "create", blend_popup)

    def edit_blend_action(self):
        """
        Command to edit a molecule blender. Set self.newblend prior to calling. This
        takes the selected blend and hands it off to the configurator to be modified.
        If the config was accepted, modify the blender in the project.
        """
        if self.blend_settings is not None:
            return
        self.blendname.set(self.newblend.name)
        self.blendvals.set(self.get_val_string_from_blend(self.newblend))
        blend_popup = tk.Toplevel(self.gui)
        self.blend_settings = BlendConfigurator(self.blendname, self.blendvals, self.blendaction, "modify", blend_popup)

    def delete_blend_action(self):
        """
        Command to delete a blend. Set self.newblend before calling. Asks
        for confirmation before deleting.
        """
        if self.blend_settings is not None:
            return
        confirm = tk.messagebox.askyesno(message='Really delete the blend definition "{}"?\nThe molecules will not be deleted.'.format(self.newblend.name))
        if confirm:
            self.gui.blend_removed(self.newblend)
            self.project.delete_blender(self.newblend)
            self.gui.molecule_modified()
        
        self.blend_settings = None
        self.newblend = None

    def get_val_string_from_blend(self, blend):
        """
        Get a string representation of the molecules in this blend
        """
        weights = {}
        for key, val in blend.molecules.items():
            weights[key.index] = val
        return json.dumps(weights)

    def set_blend_vals_from_string(self, blend, valstr):
        """
        Update the molecules in this blend using a string representation
        """
        blend.molecules = {}
        try:
            srcdic = json.loads(valstr)
        except json.decoder.JSONDecodeError:
            return
        for key, val in srcdic.items():
            for mol in self.project.molecules:
                if mol.index == int(key):
                    blend.molecules[mol] = val

    def blend_modified(self, arg1, arg2, arg3):
        """
        Determines if a blend should be modified with the new settings or if
        the changes should be discarded.
        """
        if self.blendaction.get() == "create":
            self.newblend.name = self.blendname.get()
            self.set_blend_vals_from_string(self.newblend, self.blendvals.get())
            self.gui.blend_created(self.newblend)
            self.project.add_blender(self.newblend)
            self.blend_settings = None
            self.newblend = None
        elif self.blendaction.get() == "modify":
            self.newblend.name = self.blendname.get()
            self.set_blend_vals_from_string(self.newblend, self.blendvals.get())
            self.gui.molecule_modified()
            self.blend_settings = None
            self.newblend = None
        elif self.blendaction.get() == "cancel":
            self.blend_settings = None
            self.newblend = None

    def zoom_in_action(self):
        """
        Command to increase zoom level
        """
        self.gui.drawarea.zoom_in()

    def zoom_out_action(self):
        """
        Command to decrease zoom level
        """
        self.gui.drawarea.zoom_out()

    def no_action(self):
        """
        Placeholder for buttons that have no action
        """
        print("That button doesn't do anything")

    def export_action(self):
        """
        Command to save the output PDB file
        """
        export_popup = tk.Toplevel(self.gui)
        Exporter(self.project, self.exportfilevar, self.projfilevar, export_popup)

    def import_solute_action(self):
        """
        Command to import a PDB system that will be merged with the current project
        """
        if self.project.import_solute is not None:
            self.importfilevar.set(self.project.import_solute)
        import_popup = tk.Toplevel(self.gui)
        SoluteImporter(self.project, self.importfilevar, import_popup)
